@@ -145,6 +145,7 @@ export UBSAN_OPTIONS="print_stacktrace=1 abort_on_error=1"
 # AddressSanitizer configuration
 export ASAN_OPTIONS="detect_leaks=1 abort_on_error=1 symbolize=1"
 export ASAN_SYMBOLIZER_PATH="/opt/mongodbtoolchain/v3/bin/llvm-symbolizer"
+export TSAN_OPTIONS="suppressions=./.tsan-suppressions"
 
 case "$MARCH" in
    i386)
@@ -208,6 +209,10 @@ if [ "$COMPILE_LIBMONGOCRYPT" = "ON" ]; then
    $CMAKE -DENABLE_SHARED_BSON=ON -DCMAKE_BUILD_TYPE="Debug" -DCMAKE_INSTALL_PREFIX="$INSTALL_DIR" -DCMAKE_PREFIX_PATH="$INSTALL_DIR" ../
    make install
    cd ../../
+   else
+   # Hosts may have libmongocrypt installed from apt/yum. We do not want to pick those up
+   # since those libmongocrypt packages statically link libbson.
+   CONFIGURE_FLAGS="$CONFIGURE_FLAGS -DENABLE_CLIENT_SIDE_ENCRYPTION=OFF"
 fi
 
 if [ "$ANALYZE" = "ON" ]; then
@@ -252,16 +257,18 @@ else
    }
 fi
 
+export MONGOC_TEST_SERVER_LOG=stdout
+
 # Write stderr to error.log and to console. Turn off tracing to avoid spurious
 # log messages that CHECK_LOG considers failures.
 mkfifo pipe || true
 if [ -e pipe ]; then
    set +o xtrace
    tee error.log < pipe &
-   run_valgrind ./src/libmongoc/test-libmongoc -d -F test-results.json 2>pipe
+   run_valgrind ./src/libmongoc/test-libmongoc --no-fork -d -F test-results.json 2>pipe
    rm pipe
 else
-   run_valgrind ./src/libmongoc/test-libmongoc -d -F test-results.json
+   run_valgrind ./src/libmongoc/test-libmongoc --no-fork -d -F test-results.json
 fi
 
 # Check if the error.log exists, and is more than 0 byte
