@@ -15,7 +15,7 @@
  */
 
 
-#include <bson.h>
+#include <bson/bson.h>
 
 #include "TestSuite.h"
 
@@ -391,6 +391,35 @@ test_bson_iter_init_find_w_len (void)
 }
 
 
+static void
+test_bson_iter_empty_key (void)
+{
+   /* create a bson document empty keys. */
+   bson_t *bson = BCON_NEW (
+      "", "{", "x", BCON_INT32 (1), "", "{", "y", BCON_INT32 (2), "}", "}");
+   bson_iter_t iter;
+   bson_iter_t descendant;
+
+   BSON_ASSERT (bson_iter_init_find (&iter, bson, ""));
+   BSON_ASSERT (bson_iter_init_find_case (&iter, bson, ""));
+   BSON_ASSERT (BSON_ITER_HOLDS_DOCUMENT (&iter));
+   /* tests fixes applied in CDRIVER-2755. */
+   /* a find_w_len and length 0 should also find the key. */
+   BSON_ASSERT (bson_iter_init_find_w_len (&iter, bson, "", 0));
+   BSON_ASSERT (BSON_ITER_HOLDS_DOCUMENT (&iter));
+   /* similarly, we should descend on an empty key. */
+   bson_iter_init (&iter, bson);
+   BSON_ASSERT (bson_iter_find_descendant (&iter, ".x", &descendant));
+   BSON_ASSERT (BSON_ITER_HOLDS_INT32 (&descendant));
+   ASSERT_CMPINT (bson_iter_int32 (&descendant), ==, 1);
+   bson_iter_init (&iter, bson);
+   BSON_ASSERT (bson_iter_find_descendant (&iter, "..y", &descendant));
+   BSON_ASSERT (BSON_ITER_HOLDS_INT32 (&descendant));
+   ASSERT_CMPINT (bson_iter_int32 (&descendant), ==, 2);
+   bson_destroy (bson);
+}
+
+
 static void
 test_bson_iter_as_double (void)
 {
@@ -520,6 +549,60 @@ test_bson_iter_overwrite_bool (void)
 }
 
 
+static void
+test_bson_iter_overwrite_oid (void)
+{
+   bson_oid_t oid;
+   bson_iter_t iter;
+   bson_t b;
+
+   bson_oid_init_from_string (&oid, "000000000000000000001234");
+   bson_init (&b);
+   BSON_ASSERT (BSON_APPEND_OID (&b, "key", &oid));
+   BSON_ASSERT (bson_iter_init_find (&iter, &b, "key"));
+   bson_oid_init_from_string (&oid, "000000000000000000004321");
+   bson_iter_overwrite_oid (&iter, &oid);
+   BSON_ASSERT (bson_iter_init_find (&iter, &b, "key"));
+   ASSERT_CMPOID (bson_iter_oid (&iter), &oid);
+   bson_destroy (&b);
+}
+
+
+static void
+test_bson_iter_overwrite_timestamp (void)
+{
+   bson_iter_t iter;
+   bson_t b;
+   uint32_t t, i;
+
+   bson_init (&b);
+   BSON_ASSERT (BSON_APPEND_TIMESTAMP (&b, "key", 1, 2));
+   BSON_ASSERT (bson_iter_init_find (&iter, &b, "key"));
+   bson_iter_overwrite_timestamp (&iter, 3, 4);
+   BSON_ASSERT (bson_iter_init_find (&iter, &b, "key"));
+   bson_iter_timestamp (&iter, &t, &i);
+   ASSERT_CMPUINT32 (t, ==, 3);
+   ASSERT_CMPUINT32 (i, ==, 4);
+   bson_destroy (&b);
+}
+
+
+static void
+test_bson_iter_overwrite_date_time (void)
+{
+   bson_iter_t iter;
+   bson_t b;
+
+   bson_init (&b);
+   BSON_ASSERT (BSON_APPEND_DATE_TIME (&b, "key", 1234567890));
+   BSON_ASSERT (bson_iter_init_find (&iter, &b, "key"));
+   bson_iter_overwrite_date_time (&iter, -1234567890);
+   BSON_ASSERT (bson_iter_init_find (&iter, &b, "key"));
+   ASSERT_CMPINT64 (bson_iter_date_time (&iter), ==, -1234567890);
+   bson_destroy (&b);
+}
+
+
 static void
 test_bson_iter_recurse (void)
 {
@@ -670,6 +753,14 @@ test_iter_install (TestSuite *suite)
                   test_bson_iter_overwrite_double);
    TestSuite_Add (
       suite, "/bson/iter/test_overwrite_bool", test_bson_iter_overwrite_bool);
+   TestSuite_Add (
+      suite, "/bson/iter/test_overwrite_oid", test_bson_iter_overwrite_oid);
+   TestSuite_Add (suite,
+                  "/bson/iter/test_overwrite_timestamp",
+                  test_bson_iter_overwrite_timestamp);
+   TestSuite_Add (suite,
+                  "/bson/iter/test_overwrite_date_time",
+                  test_bson_iter_overwrite_date_time);
    TestSuite_Add (suite,
                   "/bson/iter/test_bson_iter_overwrite_decimal128",
                   test_bson_iter_overwrite_decimal128);
@@ -682,4 +773,5 @@ test_iter_install (TestSuite *suite)
    TestSuite_Add (
       suite, "/bson/iter/binary_deprecated", test_bson_iter_binary_deprecated);
    TestSuite_Add (suite, "/bson/iter/from_data", test_bson_iter_from_data);
+   TestSuite_Add (suite, "/bson/iter/empty_key", test_bson_iter_empty_key);
 }
