@@ -36,7 +36,8 @@ MD_FLAGS = [
     "MONGOC_MD_FLAG_ENABLE_SHM_COUNTERS",
     "MONGOC_MD_FLAG_TRACE",
     "MONGOC_MD_FLAG_ENABLE_ICU",
-    "MONGOC_MD_FLAG_ENABLE_CLIENT_SIDE_ENCRYPTION"
+    "MONGOC_MD_FLAG_ENABLE_CLIENT_SIDE_ENCRYPTION",
+    "MONGOC_MD_FLAG_ENABLE_MONGODB_AWS_AUTH"
 ]
 
 def main():
