@@ -15,14 +15,14 @@
  */
 
 
-#include "mongoc/mongoc-bulk-operation.h"
-#include "mongoc/mongoc-bulk-operation-private.h"
-#include "mongoc/mongoc-client-private.h"
-#include "mongoc/mongoc-trace-private.h"
-#include "mongoc/mongoc-write-concern-private.h"
-#include "mongoc/mongoc-util-private.h"
-#include "mongoc/mongoc-opts-private.h"
-#include "mongoc/mongoc-write-command-private.h"
+#include "mongoc-bulk-operation.h"
+#include "mongoc-bulk-operation-private.h"
+#include "mongoc-client-private.h"
+#include "mongoc-trace-private.h"
+#include "mongoc-write-concern-private.h"
+#include "mongoc-util-private.h"
+#include "mongoc-opts-private.h"
+#include "mongoc-write-command-private.h"
 
 
 /*
@@ -148,6 +148,7 @@ _mongoc_bulk_operation_remove_with_opts (
    bson_t opts;
    bool has_collation;
    bool ret = false;
+   bool has_delete_hint;
 
    ENTRY;
 
@@ -174,11 +175,17 @@ _mongoc_bulk_operation_remove_with_opts (
       bson_append_document (&opts, "collation", 9, &remove_opts->collation);
    }
 
+   has_delete_hint = !!(remove_opts->hint.value_type);
+   if (has_delete_hint) {
+      bson_append_value (&opts, "hint", 4, &remove_opts->hint);
+   }
+
    if (bulk->commands.len) {
       last = &_mongoc_array_index (
          &bulk->commands, mongoc_write_command_t, bulk->commands.len - 1);
       if (last->type == MONGOC_WRITE_COMMAND_DELETE) {
          last->flags.has_collation |= has_collation;
+         last->flags.has_delete_hint |= has_delete_hint;
          last->flags.has_multi_write |= (remove_opts->limit == 0);
          _mongoc_write_command_delete_append (last, selector, &opts);
          ret = true;
@@ -190,6 +197,7 @@ _mongoc_bulk_operation_remove_with_opts (
       &command, selector, NULL, &opts, bulk->flags, bulk->operation_id);
 
    command.flags.has_collation = has_collation;
+   command.flags.has_delete_hint = has_delete_hint;
    command.flags.has_multi_write = (remove_opts->limit == 0);
 
    _mongoc_array_append_val (&bulk->commands, command);
@@ -378,12 +386,7 @@ mongoc_bulk_operation_insert_with_opts (mongoc_bulk_operation_t *bulk,
    }
 
    _mongoc_write_command_init_insert (
-      &command,
-      document,
-      &insert_opts.extra,
-      bulk->flags,
-      bulk->operation_id,
-      !mongoc_write_concern_is_acknowledged (bulk->write_concern));
+      &command, document, &insert_opts.extra, bulk->flags, bulk->operation_id);
 
    _mongoc_array_append_val (&bulk->commands, command);
 
@@ -408,6 +411,7 @@ _mongoc_bulk_operation_update_append (
    bson_t opts;
    bool has_collation;
    bool has_array_filters;
+   bool has_update_hint;
 
    bson_init (&opts);
    bson_append_bool (&opts, "upsert", 6, update_opts->upsert);
@@ -423,6 +427,11 @@ _mongoc_bulk_operation_update_append (
       bson_append_document (&opts, "collation", 9, &update_opts->collation);
    }
 
+   has_update_hint = !!(update_opts->hint.value_type);
+   if (has_update_hint) {
+      bson_append_value (&opts, "hint", 4, &update_opts->hint);
+   }
+
    if (extra_opts) {
       bson_concat (&opts, extra_opts);
    }
@@ -431,7 +440,9 @@ _mongoc_bulk_operation_update_append (
       last = &_mongoc_array_index (
          &bulk->commands, mongoc_write_command_t, bulk->commands.len - 1);
       if (last->type == MONGOC_WRITE_COMMAND_UPDATE) {
+         last->flags.has_array_filters |= has_array_filters;
          last->flags.has_collation |= has_collation;
+         last->flags.has_update_hint |= has_update_hint;
          last->flags.has_multi_write |= update_opts->multi;
          _mongoc_write_command_update_append (last, selector, document, &opts);
          bson_destroy (&opts);
@@ -444,6 +455,7 @@ _mongoc_bulk_operation_update_append (
 
    command.flags.has_array_filters = has_array_filters;
    command.flags.has_collation = has_collation;
+   command.flags.has_update_hint = has_update_hint;
    command.flags.has_multi_write = update_opts->multi;
 
    _mongoc_array_append_val (&bulk->commands, command);
@@ -788,6 +800,11 @@ mongoc_bulk_operation_execute (mongoc_bulk_operation_t *bulk, /* IN */
                                      &bulk->result);
 
       bulk->server_id = server_stream->sd->id;
+      /* If a retryable error occurred and a new primary was selected, use it in
+       * subsequent commands. */
+      if (bulk->result.retry_server_id) {
+         bulk->server_id = bulk->result.retry_server_id;
+      }
 
       if (bulk->result.failed &&
           (bulk->flags.ordered || bulk->result.must_stop)) {
