@@ -15,19 +15,20 @@
  */
 
 
-#include "mongoc/mongoc-cursor.h"
-#include "mongoc/mongoc-cursor-private.h"
-#include "mongoc/mongoc-client-private.h"
-#include "mongoc/mongoc-client-session-private.h"
-#include "mongoc/mongoc-counters-private.h"
-#include "mongoc/mongoc-error.h"
-#include "mongoc/mongoc-log.h"
-#include "mongoc/mongoc-trace-private.h"
-#include "mongoc/mongoc-read-concern-private.h"
-#include "mongoc/mongoc-util-private.h"
-#include "mongoc/mongoc-write-concern-private.h"
-#include "mongoc/mongoc-read-prefs-private.h"
-
+#include "mongoc-cursor.h"
+#include "mongoc-cursor-private.h"
+#include "mongoc-client-private.h"
+#include "mongoc-client-session-private.h"
+#include "mongoc-counters-private.h"
+#include "mongoc-error.h"
+#include "mongoc-error-private.h"
+#include "mongoc-log.h"
+#include "mongoc-trace-private.h"
+#include "mongoc-read-concern-private.h"
+#include "mongoc-util-private.h"
+#include "mongoc-write-concern-private.h"
+#include "mongoc-read-prefs-private.h"
+#include "mongoc-aggregate-private.h"
 
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "cursor"
@@ -163,8 +164,9 @@ _mongoc_set_cursor_ns (mongoc_cursor_t *cursor, const char *ns, uint32_t nslen)
 {
    const char *dot;
 
-   bson_strncpy (cursor->ns, ns, sizeof cursor->ns);
-   cursor->nslen = BSON_MIN (nslen, sizeof cursor->ns);
+   bson_free (cursor->ns);
+   cursor->ns = bson_strndup (ns, nslen);
+   cursor->nslen = nslen;
    dot = strstr (cursor->ns, ".");
 
    if (dot) {
@@ -586,7 +588,7 @@ done:
 void
 mongoc_cursor_destroy (mongoc_cursor_t *cursor)
 {
-   char db[MONGOC_NAMESPACE_MAX];
+   char *db;
    ENTRY;
 
    if (!cursor) {
@@ -597,17 +599,20 @@ mongoc_cursor_destroy (mongoc_cursor_t *cursor)
       cursor->impl.destroy (&cursor->impl);
    }
 
-   if (cursor->client_generation == cursor->client->generation) {
-      if (cursor->in_exhaust) {
-         cursor->client->in_exhaust = false;
-         if (cursor->state != DONE) {
-            /* The only way to stop an exhaust cursor is to kill the connection
-             */
-            mongoc_cluster_disconnect_node (
-               &cursor->client->cluster, cursor->server_id, false, NULL);
-         }
-      } else if (cursor->cursor_id) {
-         bson_strncpy (db, cursor->ns, cursor->dblen + 1);
+   /* Always close the socket for an exhaust cursor, even if the client was
+    * reset with mongoc_client_reset. That prevents further use of that socket.
+    */
+   if (cursor->in_exhaust) {
+      cursor->client->in_exhaust = false;
+      if (cursor->state != DONE) {
+         /* The only way to stop an exhaust cursor is to kill the connection
+            */
+         mongoc_cluster_disconnect_node (&cursor->client->cluster,
+                                         cursor->server_id);
+      }
+   } else if (cursor->client_generation == cursor->client->generation) {
+      if (cursor->cursor_id) {
+         db = bson_strndup (cursor->ns, cursor->dblen);
 
          _mongoc_client_kill_cursor (cursor->client,
                                      cursor->server_id,
@@ -616,6 +621,7 @@ mongoc_cursor_destroy (mongoc_cursor_t *cursor)
                                      db,
                                      cursor->ns + cursor->dblen + 1,
                                      cursor->client_session);
+         bson_free (db);
       }
    }
 
@@ -629,6 +635,7 @@ mongoc_cursor_destroy (mongoc_cursor_t *cursor)
 
    bson_destroy (&cursor->opts);
    bson_destroy (&cursor->error_doc);
+   bson_free (cursor->ns);
    bson_free (cursor);
 
    mongoc_counter_cursors_active_dec ();
@@ -684,7 +691,7 @@ _mongoc_cursor_monitor_command (mongoc_cursor_t *cursor,
 {
    mongoc_client_t *client;
    mongoc_apm_command_started_t event;
-   char db[MONGOC_NAMESPACE_MAX];
+   char *db;
 
    ENTRY;
 
@@ -694,7 +701,7 @@ _mongoc_cursor_monitor_command (mongoc_cursor_t *cursor,
       RETURN (true);
    }
 
-   bson_strncpy (db, cursor->ns, cursor->dblen + 1);
+   db = bson_strndup (cursor->ns, cursor->dblen);
 
    mongoc_apm_command_started_init (&event,
                                     cmd,
@@ -708,6 +715,7 @@ _mongoc_cursor_monitor_command (mongoc_cursor_t *cursor,
 
    client->apm_callbacks.started (&event);
    mongoc_apm_command_started_cleanup (&event);
+   bson_free (db);
 
    RETURN (true);
 }
@@ -905,22 +913,22 @@ bool
 _mongoc_cursor_run_command (mongoc_cursor_t *cursor,
                             const bson_t *command,
                             const bson_t *opts,
-                            bson_t *reply)
+                            bson_t *reply,
+                            bool retry_prohibited)
 {
-   mongoc_cluster_t *cluster;
    mongoc_server_stream_t *server_stream;
    bson_iter_t iter;
    mongoc_cmd_parts_t parts;
    const char *cmd_name;
    bool is_primary;
    mongoc_read_prefs_t *prefs = NULL;
-   char db[MONGOC_NAMESPACE_MAX];
+   char *db = NULL;
    mongoc_session_opt_t *session_opts;
    bool ret = false;
+   bool is_retryable = true;
 
    ENTRY;
 
-   cluster = &cursor->client->cluster;
    mongoc_cmd_parts_init (
       &parts, cursor->client, db, MONGOC_QUERY_NONE, command);
    parts.is_read_command = true;
@@ -982,7 +990,7 @@ _mongoc_cursor_run_command (mongoc_cursor_t *cursor,
       GOTO (done);
    }
 
-   bson_strncpy (db, cursor->ns, cursor->dblen + 1);
+   db = bson_strndup (cursor->ns, cursor->dblen);
    parts.assembled.db_name = db;
 
    if (!_mongoc_cursor_opts_to_flags (
@@ -1009,6 +1017,24 @@ _mongoc_cursor_run_command (mongoc_cursor_t *cursor,
       parts.read_prefs = cursor->read_prefs;
    }
 
+   is_retryable = _is_retryable_read (&parts, server_stream);
+   if (!strcmp (cmd_name, "getMore")) {
+      is_retryable = false;
+   }
+   if (!strcmp (cmd_name, "aggregate")) {
+      bson_iter_t pipeline_iter;
+      if (bson_iter_init_find (&pipeline_iter, command, "pipeline") &&
+          BSON_ITER_HOLDS_ARRAY (&pipeline_iter) &&
+          bson_iter_recurse (&pipeline_iter, &pipeline_iter)) {
+         if (_has_write_key (&pipeline_iter)) {
+            is_retryable = false;
+         }
+      }
+   }
+   if (is_retryable && retry_prohibited) {
+      is_retryable = false;
+   }
+
    if (cursor->write_concern &&
        !mongoc_write_concern_is_default (cursor->write_concern) &&
        server_stream->sd->max_wire_version >= WIRE_VERSION_CMD_WRITE_CONCERN) {
@@ -1022,8 +1048,35 @@ _mongoc_cursor_run_command (mongoc_cursor_t *cursor,
       GOTO (done);
    }
 
+retry:
    ret = mongoc_cluster_run_command_monitored (
-      cluster, &parts.assembled, reply, &cursor->error);
+      &cursor->client->cluster, &parts.assembled, reply, &cursor->error);
+
+   if (ret) {
+      memset (&cursor->error, 0, sizeof (bson_error_t));
+   }
+
+   if (is_retryable &&
+       _mongoc_read_error_get_type (ret, &cursor->error, reply) ==
+          MONGOC_READ_ERR_RETRY) {
+      is_retryable = false;
+
+      mongoc_server_stream_cleanup (server_stream);
+
+      server_stream = mongoc_cluster_stream_for_reads (&cursor->client->cluster,
+                                                       cursor->read_prefs,
+                                                       cursor->client_session,
+                                                       reply,
+                                                       &cursor->error);
+
+      if (server_stream &&
+          server_stream->sd->max_wire_version >= WIRE_VERSION_RETRY_READS) {
+         cursor->server_id = server_stream->sd->id;
+         parts.assembled.server_stream = server_stream;
+         bson_destroy (reply);
+         GOTO (retry);
+      }
+   }
 
    if (cursor->error.domain) {
       bson_destroy (&cursor->error_doc);
@@ -1046,6 +1099,7 @@ done:
    mongoc_server_stream_cleanup (server_stream);
    mongoc_cmd_parts_cleanup (&parts);
    mongoc_read_prefs_destroy (prefs);
+   bson_free (db);
 
    return ret;
 }
@@ -1320,7 +1374,7 @@ mongoc_cursor_clone (const mongoc_cursor_t *cursor)
    bson_copy_to (&cursor->opts, &_clone->opts);
    bson_init (&_clone->error_doc);
 
-   bson_strncpy (_clone->ns, cursor->ns, sizeof _clone->ns);
+   _clone->ns = bson_strdup (cursor->ns);
 
    /* copy the context functions by default. */
    memcpy (&_clone->impl, &cursor->impl, sizeof (cursor->impl));
@@ -1616,7 +1670,8 @@ _mongoc_cursor_response_refresh (mongoc_cursor_t *cursor,
 
    /* server replies to find / aggregate with {cursor: {id: N, firstBatch: []}},
     * to getMore command with {cursor: {id: N, nextBatch: []}}. */
-   if (_mongoc_cursor_run_command (cursor, command, opts, &response->reply) &&
+   if (_mongoc_cursor_run_command (
+          cursor, command, opts, &response->reply, false) &&
        _mongoc_cursor_start_reading_response (cursor, response)) {
       return;
    }
@@ -1629,7 +1684,6 @@ _mongoc_cursor_response_refresh (mongoc_cursor_t *cursor,
    }
 }
 
-
 void
 _mongoc_cursor_prepare_getmore_command (mongoc_cursor_t *cursor,
                                         bson_t *command)
