@@ -24,17 +24,17 @@
 #include <windows.h>
 #endif
 
-#include "mongoc/mongoc-linux-distro-scanner-private.h"
-#include "mongoc/mongoc-handshake.h"
-#include "mongoc/mongoc-handshake-compiler-private.h"
-#include "mongoc/mongoc-handshake-os-private.h"
-#include "mongoc/mongoc-handshake-private.h"
-#include "mongoc/mongoc-client.h"
-#include "mongoc/mongoc-client-private.h"
-#include "mongoc/mongoc-error.h"
-#include "mongoc/mongoc-log.h"
-#include "mongoc/mongoc-version.h"
-#include "mongoc/mongoc-util-private.h"
+#include "mongoc-linux-distro-scanner-private.h"
+#include "mongoc-handshake.h"
+#include "mongoc-handshake-compiler-private.h"
+#include "mongoc-handshake-os-private.h"
+#include "mongoc-handshake-private.h"
+#include "mongoc-client.h"
+#include "mongoc-client-private.h"
+#include "mongoc-error.h"
+#include "mongoc-log.h"
+#include "mongoc-version.h"
+#include "mongoc-util-private.h"
 
 /*
  * Global handshake data instance. Initialized at startup from mongoc_init
@@ -190,6 +190,14 @@ _mongoc_handshake_get_config_hex_string (void)
    _set_bit (bf, byte_count, MONGOC_MD_FLAG_ENABLE_ICU);
 #endif
 
+#ifdef MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION
+   _set_bit (bf, byte_count, MONGOC_MD_FLAG_ENABLE_CLIENT_SIDE_ENCRYPTION);
+#endif
+
+#ifdef MONGOC_ENABLE_MONGODB_AWS_AUTH
+   _set_bit (bf, byte_count, MONGOC_MD_FLAG_ENABLE_MONGODB_AWS_AUTH);
+#endif
+
    str = bson_string_new ("0x");
    for (i = 0; i < byte_count; i++) {
       bson_string_append_printf (str, "%02x", bf[i]);
