@@ -14,18 +14,18 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-prelude.h"
+#include "mongoc-prelude.h"
 
 #ifndef MONGOC_WRITE_COMMAND_PRIVATE_H
 #define MONGOC_WRITE_COMMAND_PRIVATE_H
 
 #include <bson/bson.h>
 
-#include "mongoc/mongoc-client.h"
-#include "mongoc/mongoc-error.h"
-#include "mongoc/mongoc-write-concern.h"
-#include "mongoc/mongoc-server-stream-private.h"
-#include "mongoc/mongoc-buffer-private.h"
+#include "mongoc-client.h"
+#include "mongoc-error.h"
+#include "mongoc-write-concern.h"
+#include "mongoc-server-stream-private.h"
+#include "mongoc-buffer-private.h"
 
 
 BSON_BEGIN_DECLS
@@ -42,12 +42,16 @@ struct _mongoc_crud_opts_t;
  * for context. */
 #define BSON_OBJECT_ALLOWANCE (16 * 1024)
 
+#define RETRYABLE_WRITE_ERROR "RetryableWriteError"
+
 struct _mongoc_bulk_write_flags_t {
    bool ordered;
    bool bypass_document_validation;
    bool has_collation;
    bool has_multi_write;
    bool has_array_filters;
+   bool has_update_hint;
+   bool has_delete_hint;
 };
 
 
@@ -58,11 +62,6 @@ typedef struct {
    mongoc_bulk_write_flags_t flags;
    int64_t operation_id;
    bson_t cmd_opts;
-   union {
-      struct {
-         bool allow_bulk_op_insert;
-      } insert;
-   } u;
 } mongoc_write_command_t;
 
 
@@ -85,6 +84,10 @@ typedef struct {
    bool must_stop; /* The stream may have been disconnected */
    bson_error_t error;
    uint32_t upsert_append_count;
+   /* If the command initially failed with a retryable write, and selected a new
+    * primary, this contains the server id of the newly selected primary. Only
+    * applies to OP_MSG. Is left at 0 if no retry occurs. */
+   uint32_t retry_server_id;
 } mongoc_write_result_t;
 
 
@@ -112,14 +115,12 @@ _mongoc_write_command_init_insert (mongoc_write_command_t *command,
                                    const bson_t *document,
                                    const bson_t *cmd_opts,
                                    mongoc_bulk_write_flags_t flags,
-                                   int64_t operation_id,
-                                   bool allow_bulk_op_insert);
+                                   int64_t operation_id);
 void
 _mongoc_write_command_init_insert_idl (mongoc_write_command_t *command,
                                        const bson_t *document,
                                        const bson_t *cmd_opts,
-                                       int64_t operation_id,
-                                       bool allow_bulk_op_insert);
+                                       int64_t operation_id);
 void
 _mongoc_write_command_init_delete (mongoc_write_command_t *command,
                                    const bson_t *selectors,
@@ -217,9 +218,7 @@ void
 _append_array_from_command (mongoc_write_command_t *command, bson_t *bson);
 
 mongoc_write_err_type_t
-_mongoc_write_error_get_type (bool cmd_ret,
-                              const bson_error_t *cmd_err,
-                              const bson_t *reply);
+_mongoc_write_error_get_type (bson_t *reply);
 
 bool
 _mongoc_write_error_update_if_unsupported_storage_engine (bool cmd_ret,
