@@ -41,6 +41,17 @@
 #include "test-libmongoc.h"
 #include "TestSuite.h"
 
+/*
+ * Prevent failing on pedantic GCC/clang warning: "ISO C forbids conversion of
+ * function pointer to object pointer type."
+ */
+#ifdef __clang__
+#pragma clang diagnostic warning "-Wpedantic"
+#elif __GNUC__ > 6
+#pragma GCC diagnostic warning "-Wpedantic"
+#elif __GNUC__ <= 6
+#pragma GCC diagnostic warning "-pedantic"
+#endif
 
 static bson_once_t once = BSON_ONCE_INIT;
 static bson_mutex_t gTestMutex;
@@ -69,7 +80,7 @@ test_msg (const char *format, ...)
 
 
 void
-test_error (const char *format, ...)
+_test_error (const char *format, ...)
 {
    va_list ap;
 
@@ -415,7 +426,7 @@ TestSuite_RunFuncInChild (TestSuite *suite, /* IN */
                        NULL,  /* Use parent's starting directory */
                        &si,
                        &pi)) {
-      test_error ("CreateProcess failed (%d).", GetLastError ());
+      test_error ("CreateProcess failed (%ld).", GetLastError ());
       bson_free (cmdline);
 
       return -1;
@@ -932,6 +943,7 @@ int
 TestSuite_Run (TestSuite *suite) /* IN */
 {
    int failures = 0;
+   int64_t start_us;
 
    if (suite->flags & TEST_HELPTEXT) {
       TestSuite_PrintHelp (suite);
@@ -952,6 +964,7 @@ TestSuite_Run (TestSuite *suite) /* IN */
       }
    }
 
+   start_us = bson_get_monotonic_time ();
    if (suite->tests) {
       if (suite->testname) {
          failures += TestSuite_RunNamed (suite, suite->testname);
@@ -964,6 +977,8 @@ TestSuite_Run (TestSuite *suite) /* IN */
          TestSuite_PrintJsonFooter (suite->outfile);
       }
    }
+   MONGOC_DEBUG ("Duration of all tests (s): %" PRId64,
+                 (bson_get_monotonic_time () - start_us) / (1000 * 1000));
 
    return failures;
 }
@@ -1058,9 +1073,10 @@ test_suite_mock_server_log (const char *msg, ...)
 }
 
 bool
-TestSuite_NoFork (TestSuite *suite) {
+TestSuite_NoFork (TestSuite *suite)
+{
    if (suite->flags & TEST_NOFORK) {
       return true;
    }
    return false;
-}
\ No newline at end of file
+}
