@@ -19,7 +19,22 @@
 
 struct _json_test_config_t;
 
+#include "mongoc/mongoc-thread-private.h"
+#include "common-thread-private.h"
+#include "mock_server/sync-queue.h"
+
+struct _json_test_ctx_t;
+
 typedef struct {
+   struct _json_test_ctx_t *ctx;
+   bson_thread_t thread;
+   sync_queue_t *queue;
+   bson_mutex_t mutex;
+   mongoc_cond_t cond;
+   bool shutdown_requested;
+} json_test_worker_thread_t;
+
+typedef struct _json_test_ctx_t {
    const struct _json_test_config_t *config;
    uint32_t n_events;
    bson_t events;
@@ -35,6 +50,20 @@ typedef struct {
    mongoc_change_stream_t *change_stream;
    /* Sessions tests check the most recently sent two lsid's */
    bson_t *sent_lsids[2];
+   bson_mutex_t mutex;
+   /* The total number of times a server description was marked unknown. */
+   int64_t total_ServerMarkedUnknownEvent;
+   /* How many were accounted for in the test. */
+   int64_t measured_ServerMarkedUnknownEvent;
+   /* How many connection generation increments were accounted for in the test.
+    */
+   int64_t measured_PoolClearedEvent;
+   mongoc_host_list_t primary_host;
+
+   int64_t total_PrimaryChangedEvent;
+   int64_t measured_PrimaryChangedEvent;
+
+   json_test_worker_thread_t *worker_threads[2];
 } json_test_ctx_t;
 
 mongoc_client_session_t *
@@ -78,4 +107,10 @@ check_result (const bson_t *test,
               const bson_value_t *result,
               const bson_error_t *error);
 
+json_test_worker_thread_t *
+worker_thread_new (json_test_ctx_t *ctx);
+
+void
+worker_thread_destroy (json_test_worker_thread_t *wt);
+
 #endif
