@@ -72,8 +72,13 @@ struct _autoresponder_handle_t {
    int id;
 };
 
+typedef enum {
+   REPLY, HANGUP, RESET
+} reply_type_t;
+
 
 typedef struct {
+   reply_type_t type;
    mongoc_reply_flags_t flags;
    bson_t *docs;
    int n_docs;
@@ -81,15 +86,14 @@ typedef struct {
    uint16_t client_port;
    mongoc_opcode_t request_opcode;
    mongoc_query_flags_t query_flags;
+   mongoc_op_msg_flags_t opmsg_flags;
    int32_t response_to;
 } reply_t;
 
 
-static void *
-main_thread (void *data);
+static BSON_THREAD_FUN (main_thread, data);
 
-static void *
-worker_thread (void *data);
+static BSON_THREAD_FUN (worker_thread, data);
 
 static void
 _mock_server_reply_with_stream (mock_server_t *server,
@@ -319,6 +323,7 @@ mock_server_run (mock_server_t *server)
    int optval;
    uint16_t bound_port;
    size_t bind_addr_len = 0;
+   int r;
 
    MONGOC_INFO ("Starting mock server on port %d.", server->port);
    ssock = mongoc_socket_new (
@@ -382,7 +387,9 @@ mock_server_run (mock_server_t *server)
       bound_port);
    server->uri = mongoc_uri_new (server->uri_str);
 
-   bson_thread_create (&server->main_thread, main_thread, (void *) server);
+   r = COMMON_PREFIX (thread_create) (
+      &server->main_thread, main_thread, (void *) server);
+   BSON_ASSERT (r == 0);
    while (!server->running) {
       mongoc_cond_wait (&server->cond, &server->mutex);
    }
@@ -914,8 +921,8 @@ _mock_server_receives_msg (mock_server_t *server, uint32_t flags, ...)
  *
  * mock_server_receives_ismaster --
  *
- *       Pop a client ismaster call if one is enqueued, or wait up to
- *       request_timeout_ms for the client to send a request.
+ *       Pop a client non-streaming ismaster call if one is enqueued,
+ *       or wait up to request_timeout_ms for the client to send a request.
  *
  * Returns:
  *       A request you must request_destroy, or NULL if the current
@@ -931,7 +938,10 @@ request_t *
 mock_server_receives_ismaster (mock_server_t *server)
 {
    return mock_server_receives_command (
-      server, "admin", MONGOC_QUERY_SLAVE_OK, "{'isMaster': 1}");
+      server,
+      "admin",
+      MONGOC_QUERY_SLAVE_OK,
+      "{'isMaster': 1, 'maxAwaitTimeMS': { '$exists': false }}");
 }
 
 
@@ -1217,12 +1227,14 @@ mock_server_receives_kill_cursors (mock_server_t *server, int64_t cursor_id)
 void
 mock_server_hangs_up (request_t *request)
 {
+   reply_t *reply;
    test_suite_mock_server_log ("%5.2f  %hu <- %hu \thang up!",
                                mock_server_get_uptime_sec (request->server),
                                request->client_port,
                                request_get_server_port (request));
-
-   mongoc_stream_close (request->client);
+   reply = bson_malloc0 (sizeof (reply_t));
+   reply->type = HANGUP;
+   q_put (request->replies, reply);
 }
 
 
@@ -1244,20 +1256,15 @@ mock_server_hangs_up (request_t *request)
 void
 mock_server_resets (request_t *request)
 {
-   struct linger no_linger;
-   no_linger.l_onoff = 1;
-   no_linger.l_linger = 0;
-
+   reply_t *reply;
    test_suite_mock_server_log ("%5.2f  %hu <- %hu \treset!",
                                mock_server_get_uptime_sec (request->server),
                                request->client_port,
                                request_get_server_port (request));
 
-   /* send RST packet to client */
-   mongoc_stream_setsockopt (
-      request->client, SOL_SOCKET, SO_LINGER, &no_linger, sizeof no_linger);
-
-   mongoc_stream_close (request->client);
+   reply = bson_malloc0 (sizeof (reply_t));
+   reply->type = RESET;
+   q_put (request->replies, reply);
 }
 
 
@@ -1330,6 +1337,31 @@ mock_server_replies_simple (request_t *request, const char *docs_json)
    mock_server_replies (request, MONGOC_REPLY_NONE, 0, 0, 1, docs_json);
 }
 
+/* To specify additional flags for OP_MSG replies. */
+void
+mock_server_replies_opmsg (request_t *request,
+                           mongoc_op_msg_flags_t flags,
+                           const bson_t *doc)
+{
+   reply_t *reply;
+
+   BSON_ASSERT (request);
+
+   reply = bson_malloc0 (sizeof (reply_t));
+
+   reply->opmsg_flags = flags;
+   reply->n_docs = 1;
+   reply->docs = bson_malloc0 (sizeof (bson_t));
+   bson_copy_to (doc, &reply->docs[0]);
+
+   reply->cursor_id = 0;
+   reply->client_port = request_get_client_port (request);
+   reply->request_opcode = MONGOC_OPCODE_MSG;
+   reply->response_to = request->request_rpc.header.request_id;
+
+   q_put (request->replies, reply);
+}
+
 
 /*--------------------------------------------------------------------------
  *
@@ -1381,9 +1413,9 @@ mock_server_replies_to_find (request_t *request,
                              bool is_command)
 {
    char *find_reply;
-   char db[MONGOC_NAMESPACE_MAX];
+   char *db;
 
-   _mongoc_get_db_name (ns, db);
+   db = _mongoc_get_db_name (ns);
 
    /* minimal validation, we're not testing query / find cmd here */
    if (request->is_command && !is_command) {
@@ -1417,6 +1449,7 @@ mock_server_replies_to_find (request_t *request,
       mock_server_replies (
          request, MONGOC_REPLY_NONE, cursor_id, 0, number_returned, reply_json);
    }
+   bson_free (db);
 }
 
 
@@ -1444,6 +1477,10 @@ mock_server_destroy (mock_server_t *server)
    int64_t deadline = bson_get_monotonic_time () + 10 * 1000 * 1000;
    request_t *request;
 
+   if (!server) {
+      return;
+   }
+
    bson_mutex_lock (&server->mutex);
    if (server->running) {
       server->stopped = true;
@@ -1470,7 +1507,7 @@ mock_server_destroy (mock_server_t *server)
    }
 
    bson_mutex_unlock (&server->mutex);
-   bson_thread_join (server->main_thread);
+   COMMON_PREFIX (thread_join) (server->main_thread);
 
    _mongoc_array_destroy (&server->worker_threads);
 
@@ -1538,8 +1575,7 @@ typedef struct worker_closure_t {
 } worker_closure_t;
 
 
-static void *
-main_thread (void *data)
+static BSON_THREAD_FUN (main_thread, data)
 {
    mock_server_t *server = (mock_server_t *) data;
    mongoc_socket_t *client_sock;
@@ -1549,6 +1585,7 @@ main_thread (void *data)
    bson_thread_t thread;
    mongoc_array_t worker_threads;
    size_t i;
+   int r;
 
    bson_mutex_lock (&server->mutex);
    server->running = true;
@@ -1596,7 +1633,8 @@ main_thread (void *data)
          closure->port = port;
 
          bson_mutex_lock (&server->mutex);
-         bson_thread_create (&thread, worker_thread, closure);
+         r = COMMON_PREFIX (thread_create) (&thread, worker_thread, closure);
+         BSON_ASSERT (r == 0);
          _mongoc_array_append_val (&server->worker_threads, thread);
          bson_mutex_unlock (&server->mutex);
       }
@@ -1609,8 +1647,8 @@ main_thread (void *data)
    bson_mutex_unlock (&server->mutex);
 
    for (i = 0; i < worker_threads.len; i++) {
-      bson_thread_join (
-         _mongoc_array_index (&worker_threads, bson_thread_t, i));
+      COMMON_PREFIX (thread_join)
+      (_mongoc_array_index (&worker_threads, bson_thread_t, i));
    }
 
    _mongoc_array_destroy (&worker_threads);
@@ -1619,7 +1657,7 @@ main_thread (void *data)
    server->running = false;
    bson_mutex_unlock (&server->mutex);
 
-   return NULL;
+   BSON_THREAD_RETURN;
 }
 
 
@@ -1637,8 +1675,7 @@ _reply_destroy (reply_t *reply)
 }
 
 
-static void *
-worker_thread (void *data)
+static BSON_THREAD_FUN (worker_thread, data)
 {
    worker_closure_t *closure = (worker_closure_t *) data;
    mock_server_t *server = closure->server;
@@ -1677,7 +1714,7 @@ worker_thread (void *data)
          mongoc_stream_destroy (client_stream);
          bson_free (closure);
          q_destroy (replies);
-         RETURN (NULL);
+         BSON_THREAD_RETURN;
       }
    }
 #endif
@@ -1780,7 +1817,7 @@ failure:
 
    q_destroy (replies);
 
-   RETURN (NULL);
+   BSON_THREAD_RETURN;
 }
 
 
@@ -1799,6 +1836,7 @@ mock_server_reply_multi (request_t *request,
 
    reply = bson_malloc0 (sizeof (reply_t));
 
+   reply->type = REPLY;
    reply->flags = flags;
    reply->n_docs = n_docs;
    reply->docs = bson_malloc0 (n_docs * sizeof (bson_t));
@@ -1835,12 +1873,27 @@ _mock_server_reply_with_stream (mock_server_t *server,
    uint8_t *ptr;
    size_t len;
    bool is_op_msg;
-
    mongoc_reply_flags_t flags = reply->flags;
    const bson_t *docs = reply->docs;
    int n_docs = reply->n_docs;
    int64_t cursor_id = reply->cursor_id;
 
+   if (reply->type == HANGUP) {
+      mongoc_stream_close (client);
+      return;
+   } else if (reply->type == RESET) {
+      struct linger no_linger;
+      no_linger.l_onoff = 1;
+      no_linger.l_linger = 0;
+
+      /* send RST packet to client */
+      mongoc_stream_setsockopt (
+         client, SOL_SOCKET, SO_LINGER, &no_linger, sizeof no_linger);
+
+      mongoc_stream_close (client);
+      return;
+   }
+
    docs_json = bson_string_new ("");
    for (i = 0; i < n_docs; i++) {
       doc_json = bson_as_json (&docs[i], NULL);
@@ -1890,6 +1943,7 @@ _mock_server_reply_with_stream (mock_server_t *server,
    if (is_op_msg) {
       r.header.opcode = MONGOC_OPCODE_MSG;
       r.msg.n_sections = 1;
+      r.msg.flags = reply->opmsg_flags;
       /* we don't yet implement payload type 1, a document stream */
       r.msg.sections[0].payload_type = 0;
       r.msg.sections[0].payload.bson_document = buf;
