@@ -19,6 +19,7 @@
 
 #include <bson/bson.h>
 
+#include "mongoc/mongoc-flags-private.h"
 #include "mongoc/mongoc-uri.h"
 
 #ifdef MONGOC_ENABLE_SSL
@@ -199,6 +200,11 @@ mock_server_replies_to_find (request_t *request,
                              const char *reply_json,
                              bool is_command);
 
+void
+mock_server_replies_opmsg (request_t *request,
+                           mongoc_op_msg_flags_t flags,
+                           const bson_t *doc);
+
 void
 mock_server_reply_multi (request_t *request,
                          mongoc_reply_flags_t flags,
@@ -216,7 +222,9 @@ rs_response_to_ismaster (mock_server_t *server,
                          int has_tags,
                          ...);
 
-#define RS_RESPONSE_TO_ISMASTER(server, primary, has_tags, ...) \
-   rs_response_to_ismaster (server, primary, has_tags, __VA_ARGS__, NULL)
+#define RS_RESPONSE_TO_ISMASTER(                     \
+   server, max_wire_version, primary, has_tags, ...) \
+   rs_response_to_ismaster (                         \
+      server, max_wire_version, primary, has_tags, __VA_ARGS__, NULL)
 
 #endif /* MOCK_SERVER_H */
