@@ -28,6 +28,7 @@
 #include "mongoc/mongoc-client-private.h"
 
 #include "test-conveniences.h"
+#include "test-libmongoc.h"
 #include "TestSuite.h"
 
 #ifdef BSON_HAVE_STRINGS_H
@@ -314,6 +315,10 @@ bson_lookup_read_concern (const bson_t *b, const char *key)
    mongoc_read_concern_t *rc = mongoc_read_concern_new ();
 
    bson_lookup_doc (b, key, &doc);
+   if (!bson_has_field (&doc, "level")) {
+      /* empty document means default read concern. */
+      return rc;
+   }
    mongoc_read_concern_set_level (rc, bson_lookup_utf8 (&doc, "level"));
 
    return rc;
@@ -340,10 +345,10 @@ bson_lookup_write_concern (const bson_t *b, const char *key)
    bson_lookup_doc (b, key, &doc);
    BSON_ASSERT (bson_iter_init (&iter, &doc));
 
-   /* current command monitoring tests always have "w" but may also have
-    * "wtimeout" and "j" fields. */
-   ASSERT_CMPUINT32 (bson_count_keys (&doc), >=, (uint32_t) 1);
-   BSON_ASSERT (bson_iter_find_descendant (&iter, "w", &w));
+   if (!bson_iter_find_descendant (&iter, "w", &w)) {
+      /* empty document means default write concern. */
+      return wc;
+   }
 
    if (BSON_ITER_HOLDS_NUMBER (&w)) {
       mongoc_write_concern_set_w (wc, (int32_t) bson_iter_as_int64 (&w));
@@ -869,7 +874,7 @@ match_bson_with_ctx (const bson_t *doc, const bson_t *pattern, match_ctx_t *ctx)
    bool is_type_operator;
    bool exists;
    bool empty;
-   bson_type_t bson_type;
+   bson_type_t bson_type = (bson_type_t) 0;
    bool found;
    bson_iter_t doc_iter;
    bson_value_t doc_value;
@@ -1793,3 +1798,25 @@ json_with_all_types ()
                       "}";
    return json;
 }
+
+void
+assert_wc_oob_error (bson_error_t *error)
+{
+   if (test_framework_get_server_version () >=
+       test_framework_str_to_version ("4.3.3")) {
+      /* Error reporting changed in SERVER-45584 */
+      ASSERT_ERROR_CONTAINS (
+         (*error),
+         MONGOC_ERROR_SERVER,
+         9,
+         "w has to be a non-negative number and not greater than 50");
+   } else {
+      if (test_framework_is_replset ()) { /* replset */
+         ASSERT_ERROR_CONTAINS (
+            (*error), MONGOC_ERROR_WRITE_CONCERN, 100, "Write Concern error:");
+      } else { /* standalone */
+         ASSERT_CMPINT (error->domain, ==, MONGOC_ERROR_SERVER);
+         ASSERT_CMPINT (error->code, ==, 2);
+      }
+   }
+}
