@@ -34,6 +34,14 @@
 #include <strings.h>
 #endif
 
+#if defined(_MSC_VER) && defined(_WIN64)
+#include <errhandlingapi.h>
+#include <DbgHelp.h>
+#endif
+
+#ifdef MONGOC_ENABLE_SSL_OPENSSL
+#include "mongoc/mongoc-openssl-private.h"
+#endif
 /* libbson */
 
 
@@ -75,6 +83,8 @@ extern void
 test_value_install (TestSuite *suite);
 extern void
 test_writer_install (TestSuite *suite);
+extern void
+test_b64_install (TestSuite *suite);
 
 /* libmongoc */
 
@@ -95,8 +105,12 @@ test_client_install (TestSuite *suite);
 extern void
 test_client_max_staleness_install (TestSuite *suite);
 extern void
+test_client_hedged_reads_install (TestSuite *suite);
+extern void
 test_client_pool_install (TestSuite *suite);
 extern void
+test_client_cmd_install (TestSuite *suite);
+extern void
 test_cluster_install (TestSuite *suite);
 extern void
 test_collection_install (TestSuite *suite);
@@ -175,6 +189,8 @@ test_opts_install (TestSuite *suite);
 extern void
 test_socket_install (TestSuite *suite);
 extern void
+test_speculative_auth_install (TestSuite *suite);
+extern void
 test_stream_install (TestSuite *suite);
 extern void
 test_thread_install (TestSuite *suite);
@@ -224,6 +240,24 @@ extern void
 test_apm_install (TestSuite *suite);
 extern void
 test_client_side_encryption_install (TestSuite *suite);
+extern void
+test_long_namespace_install (TestSuite *suite);
+extern void
+test_server_description_install (TestSuite *suite);
+extern void
+test_aws_install (TestSuite *suite);
+extern void
+test_streamable_ismaster_install (TestSuite *suite);
+#if defined(MONGOC_ENABLE_OCSP_OPENSSL) && OPENSSL_VERSION_NUMBER >= 0x10101000L
+extern void
+test_ocsp_cache_install (TestSuite *suite);
+#endif
+extern void
+test_interrupt_install (TestSuite *suite);
+extern void
+test_monitoring_install (TestSuite *suite);
+extern void
+test_http_install (TestSuite *suite);
 
 typedef struct {
    mongoc_log_level_t level;
@@ -383,17 +417,21 @@ log_handler (mongoc_log_level_t log_level,
    suite = (TestSuite *) user_data;
 
    if (log_level < MONGOC_LOG_LEVEL_INFO) {
+      bson_mutex_lock (&captured_logs_mutex);
       if (capturing_logs) {
          log_entry = log_entry_create (log_level, message);
-         bson_mutex_lock (&captured_logs_mutex);
          _mongoc_array_append_val (&captured_logs, log_entry);
          bson_mutex_unlock (&captured_logs_mutex);
          return;
       }
+      bson_mutex_unlock (&captured_logs_mutex);
 
       if (!suite->silent) {
          mongoc_log_default_handler (log_level, log_domain, message, NULL);
       }
+   } else if (log_level == MONGOC_LOG_LEVEL_DEBUG &&
+              test_suite_debug_output ()) {
+      mongoc_log_default_handler (log_level, log_domain, message, NULL);
    }
 }
 
@@ -410,8 +448,8 @@ gen_collection_name (const char *str)
 {
    return bson_strdup_printf ("%s_%u_%u",
                               str,
-                              (unsigned) bson_get_monotonic_time (),
-                              (unsigned) gettestpid ());
+                              (uint32_t) bson_get_monotonic_time (),
+                              (uint32_t) gettestpid ());
 }
 
 
@@ -429,41 +467,33 @@ get_test_collection (mongoc_client_t *client, const char *prefix)
 }
 
 
-/*
- *--------------------------------------------------------------------------
- *
- * test_framework_get_env --
- *
- *       Get the value of an environment variable.
- *
- * Returns:
- *       A string you must bson_free, or NULL if the variable is not set.
- *
- * Side effects:
- *       None.
- *
- *--------------------------------------------------------------------------
- */
 char *
 test_framework_getenv (const char *name)
 {
-#ifdef _MSC_VER
-   char buf[1024];
-   size_t buflen;
+   return _mongoc_getenv (name);
+}
 
-   if ((0 == getenv_s (&buflen, buf, sizeof buf, name)) && buflen) {
-      return bson_strdup (buf);
-   } else {
-      return NULL;
+/* Returns false if unable to set environment variable. Which may occur if
+ * test-libmongoc lacks permissions to do so. */
+bool
+test_framework_setenv (const char *name, const char *value)
+{
+#ifdef _WIN32
+   char *envstring;
+
+   envstring = bson_strdup_printf ("%s=%s", name, value);
+   if (0 != _putenv (envstring)) {
+      return false;
    }
+
+   return true;
 #else
 
-   if (getenv (name) && strlen (getenv (name))) {
-      return bson_strdup (getenv (name));
-   } else {
-      return NULL;
+   if (0 != setenv (name, value, 1)) {
+      return false;
    }
 
+   return true;
 #endif
 }
 
@@ -1913,9 +1943,9 @@ int
 test_framework_skip_if_windows (void)
 {
 #ifdef _WIN32
-   return false;
+   return 0;
 #else
-   return true;
+   return 1;
 #endif
 }
 
@@ -2211,6 +2241,7 @@ WIRE_VERSION_CHECKS (5)
 WIRE_VERSION_CHECKS (6)
 WIRE_VERSION_CHECKS (7)
 WIRE_VERSION_CHECKS (8)
+WIRE_VERSION_CHECKS (9)
 
 int
 test_framework_skip_if_no_dual_ip_hostname (void)
@@ -2270,6 +2301,10 @@ test_framework_skip_if_no_failpoint (void)
    bool ret;
    bson_error_t error;
 
+   if (!TestSuite_CheckLive ()) {
+      return 0;
+   }
+
    client = test_framework_client_new ();
    mongoc_client_set_error_api (client, MONGOC_ERROR_API_VERSION_2);
    ret = mongoc_client_command_simple (
@@ -2316,6 +2351,32 @@ test_framework_skip_if_no_client_side_encryption (void)
    return 0; /* 0 == do not proceed. */
 }
 
+int
+test_framework_skip_if_no_aws (void)
+{
+#ifdef MONGOC_ENABLE_MONGODB_AWS_AUTH
+   return 1; /* proceed. */
+#else
+   return 0;    /* do not proceed. */
+#endif
+}
+
+/* test-libmongoc may not have permissions to set environment variables. */
+int
+test_framework_skip_if_no_setenv (void)
+{
+   char *value;
+   if (!test_framework_setenv ("MONGOC_TEST_CANARY", "VALUE")) {
+      return 0; /* do not proceed. */
+   }
+   value = test_framework_getenv ("MONGOC_TEST_CANARY");
+   if (!value || 0 != strcmp (value, "VALUE")) {
+      return 0; /* do not proceed. */
+   }
+   bson_free (value);
+   return 1;
+}
+
 void
 test_framework_resolve_path (const char *path, char *resolved)
 {
@@ -2326,14 +2387,147 @@ test_framework_resolve_path (const char *path, char *resolved)
    }
 }
 
+int
+test_framework_skip_if_time_sensitive (void)
+{
+   if (test_suite_valgrind ()) {
+      return 0;
+   }
+
+/* Skip time sensitive tests on macOS per CDRIVER-3549. */
+#ifdef __APPLE__
+   return 0;
+#else
+   return 1;
+#endif
+}
+
+int
+test_framework_skip_due_to_cdriver3708 (void)
+{
+   if (0 == test_framework_skip_if_auth () &&
+       0 == test_framework_skip_if_replset () &&
+       test_framework_get_server_version () >
+          test_framework_str_to_version ("4.4.0")) {
+      /* If auth is enabled, we're using a replica set, and using a > 4.4
+       * server, skip test. */
+      return 0;
+   }
+   return 1;
+}
+
 static char MONGOC_TEST_UNIQUE[32];
 
+#if defined(_MSC_VER) && defined(_WIN64)
+LONG WINAPI
+windows_exception_handler (EXCEPTION_POINTERS *pExceptionInfo)
+{
+   HANDLE process = GetCurrentProcess ();
+
+   fprintf (stderr, "entering windows exception handler\n");
+   SymInitialize (process, NULL, TRUE);
+
+   /* Shamelessly stolen from https://stackoverflow.com/a/28115589 */
+
+   /* StackWalk64() may modify context record passed to it, so we will
+    use a copy.
+    */
+   CONTEXT context_record = *pExceptionInfo->ContextRecord;
+   DWORD exception_code = pExceptionInfo->ExceptionRecord->ExceptionCode;
+   /* Initialize stack walking. */
+   char exception_string[128];
+   bson_snprintf (exception_string,
+                  sizeof (exception_string),
+                  (exception_code == EXCEPTION_ACCESS_VIOLATION)
+                     ? "(access violation)"
+                     : "0x%08X",
+                  exception_code);
+
+   char address_string[32];
+   bson_snprintf (address_string,
+                  sizeof (address_string),
+                  "0x%p",
+                  pExceptionInfo->ExceptionRecord->ExceptionAddress);
+
+   fprintf (stderr,
+            "exception '%s' at '%s', terminating\n",
+            exception_string,
+            address_string);
+
+   STACKFRAME64 stack_frame;
+   memset (&stack_frame, 0, sizeof (stack_frame));
+#if defined(_WIN64)
+   int machine_type = IMAGE_FILE_MACHINE_AMD64;
+   stack_frame.AddrPC.Offset = context_record.Rip;
+   stack_frame.AddrFrame.Offset = context_record.Rbp;
+   stack_frame.AddrStack.Offset = context_record.Rsp;
+#else
+   int machine_type = IMAGE_FILE_MACHINE_I386;
+   stack_frame.AddrPC.Offset = context_record.Eip;
+   stack_frame.AddrFrame.Offset = context_record.Ebp;
+   stack_frame.AddrStack.Offset = context_record.Esp;
+#endif
+   stack_frame.AddrPC.Mode = AddrModeFlat;
+   stack_frame.AddrFrame.Mode = AddrModeFlat;
+   stack_frame.AddrStack.Mode = AddrModeFlat;
+
+   SYMBOL_INFO *symbol;
+   symbol = calloc (sizeof (SYMBOL_INFO) + 256, 1);
+   symbol->MaxNameLen = 255;
+   symbol->SizeOfStruct = sizeof (SYMBOL_INFO);
+
+   fprintf (stderr, "begin stack trace\n");
+   while (StackWalk64 (machine_type,
+                       GetCurrentProcess (),
+                       GetCurrentThread (),
+                       &stack_frame,
+                       &context_record,
+                       NULL,
+                       &SymFunctionTableAccess64,
+                       &SymGetModuleBase64,
+                       NULL)) {
+      DWORD64 displacement = 0;
+
+      if (SymFromAddr (process,
+                       (DWORD64) stack_frame.AddrPC.Offset,
+                       &displacement,
+                       symbol)) {
+         IMAGEHLP_MODULE64 moduleInfo;
+         memset (&moduleInfo, 0, sizeof (moduleInfo));
+         moduleInfo.SizeOfStruct = sizeof (moduleInfo);
+
+         if (SymGetModuleInfo64 (process, symbol->ModBase, &moduleInfo))
+            fprintf (stderr, "%s : ", moduleInfo.ModuleName);
+
+         fprintf (stderr, "%s", symbol->Name);
+      }
+
+      IMAGEHLP_LINE line;
+      line.SizeOfStruct = sizeof (IMAGEHLP_LINE);
+
+      DWORD offset_ln = 0;
+      if (SymGetLineFromAddr (
+             process, (DWORD64) stack_frame.AddrPC.Offset, &offset_ln, &line)) {
+         fprintf (stderr, " %s:%d ", line.FileName, line.LineNumber);
+      }
+
+      fprintf (stderr, "\n");
+   }
+   fprintf (stderr, "end stack trace\n");
+   fflush (stderr);
+   return EXCEPTION_EXECUTE_HANDLER;
+}
+#endif
+
 int
 main (int argc, char *argv[])
 {
    TestSuite suite;
    int ret;
 
+#if defined(_MSC_VER) && defined(_WIN64)
+   SetUnhandledExceptionFilter (windows_exception_handler);
+#endif
    mongoc_init ();
 
    bson_snprintf (MONGOC_TEST_UNIQUE,
@@ -2375,6 +2569,7 @@ main (int argc, char *argv[])
    test_utf8_install (&suite);
    test_value_install (&suite);
    test_writer_install (&suite);
+   test_b64_install (&suite);
 
    /* libmongoc */
 
@@ -2385,7 +2580,9 @@ main (int argc, char *argv[])
    test_change_stream_install (&suite);
    test_client_install (&suite);
    test_client_max_staleness_install (&suite);
+   test_client_hedged_reads_install (&suite);
    test_client_pool_install (&suite);
+   test_client_cmd_install (&suite);
    test_write_command_install (&suite);
    test_bulk_install (&suite);
    test_cluster_install (&suite);
@@ -2406,6 +2603,7 @@ main (int argc, char *argv[])
    test_linux_distro_scanner_install (&suite);
    test_list_install (&suite);
    test_log_install (&suite);
+   test_long_namespace_install (&suite);
    test_matcher_install (&suite);
    test_mongos_pinning_install (&suite);
    test_queue_install (&suite);
@@ -2430,6 +2628,7 @@ main (int argc, char *argv[])
    test_server_selection_errors_install (&suite);
    test_session_install (&suite);
    test_set_install (&suite);
+   test_speculative_auth_install (&suite);
    test_stream_install (&suite);
    test_thread_install (&suite);
    test_topology_install (&suite);
@@ -2453,6 +2652,15 @@ main (int argc, char *argv[])
    test_crud_install (&suite);
    test_apm_install (&suite);
    test_client_side_encryption_install (&suite);
+   test_server_description_install (&suite);
+   test_aws_install (&suite);
+   test_streamable_ismaster_install (&suite);
+#if defined(MONGOC_ENABLE_OCSP_OPENSSL) && OPENSSL_VERSION_NUMBER >= 0x10101000L
+   test_ocsp_cache_install (&suite);
+#endif
+   test_interrupt_install (&suite);
+   test_monitoring_install (&suite);
+   test_http_install (&suite);
 
    ret = TestSuite_Run (&suite);
 
