@@ -45,6 +45,8 @@ bool
 test_framework_getenv_bool (const char *name);
 int64_t
 test_framework_getenv_int64 (const char *name, int64_t default_value);
+bool
+test_framework_setenv (const char *name, const char *value);
 char *
 test_framework_get_host (void);
 uint16_t
@@ -166,6 +168,7 @@ WIRE_VERSION_CHECK_DECLS (5)
 WIRE_VERSION_CHECK_DECLS (6)
 WIRE_VERSION_CHECK_DECLS (7)
 WIRE_VERSION_CHECK_DECLS (8)
+WIRE_VERSION_CHECK_DECLS (9)
 
 #undef WIRE_VERSION_CHECK_DECLS
 
@@ -204,7 +207,16 @@ test_framework_skip_if_no_failpoint (void);
 int
 test_framework_skip_if_no_client_side_encryption (void);
 
+int
+test_framework_skip_if_time_sensitive (void);
+
 void
 test_framework_resolve_path (const char *path, char *resolved);
 
+int
+test_framework_skip_if_no_aws (void);
+
+int
+test_framework_skip_if_no_setenv (void);
+
 #endif
