@@ -14,6 +14,17 @@
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "session-test"
 
+/*
+ * Prevent failing on pedantic GCC/clang warning: "ISO C forbids conversion of
+ * function pointer to object pointer type."
+ */
+#ifdef __clang__
+#pragma clang diagnostic warning "-Wpedantic"
+#elif __GNUC__ > 6
+#pragma GCC diagnostic warning "-Wpedantic"
+#elif __GNUC__ <= 6
+#pragma GCC diagnostic warning "-pedantic"
+#endif
 
 static void
 test_session_opts_clone (void)
@@ -497,6 +508,55 @@ test_mock_end_sessions_pooled (void)
    _test_mock_end_sessions (true);
 }
 
+/* Test for CDRIVER-3587 - Do not reuse server stream that becomes invalid on
+ * failure to end session */
+static void
+test_mock_end_sessions_server_disconnect (void)
+{
+   mock_server_t *server;
+   mongoc_client_t *client;
+   bson_error_t error;
+   mongoc_client_session_t *session[12000];
+   future_t *future;
+   uint16_t i;
+
+   server = mock_mongos_new (WIRE_VERSION_OP_MSG);
+   mock_server_run (server);
+
+   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+
+   for (i = 0; i < 12000; i++) {
+      session[i] = mongoc_client_start_session (client, NULL, &error);
+      ASSERT_OR_PRINT (session[i], error);
+   }
+
+   /* Simulate server failure or network failure.  Destroy the mock server here
+    * rather than at the end of the test so that the 'endSessions' commands fail
+    * to reach the mock server. */
+   mock_server_destroy (server);
+
+   /* The below calls to mongoc_client_session_destroy () will produce a warning
+    * regarding the inability to send the 'endSessions' command. */
+   capture_logs (true);
+
+   for (i = 0; i < 12000; i++) {
+      mongoc_client_session_destroy (session[i]);
+   }
+
+   /* The above loop will add each session back to the session pool.  If
+    * CDRIVER-3587 has not been fixed, the mongoc_client_destroy () call below
+    * will create 'endSessions' commands which will be sent but fail to reach
+    * the server; the associated server stream will not be correctly
+    * invalidated.  Subsequent reuse of the stream, as in the attempt to send
+    * the second batch of 10,000 during the attempt to destroy the client, will
+    * trigger a segfault. */
+
+   future = future_client_destroy (client);
+
+   future_wait (future);
+   future_destroy (future);
+}
+
 typedef struct {
    int started_calls;
    int succeeded_calls;
@@ -696,6 +756,32 @@ test_end_sessions_pooled (void *ctx)
    _test_end_sessions (true);
 }
 
+/* Sends ping to server via client_session.  useful for marking
+ * server_sessions as used so that they are pushed back to the session pool */
+static void
+send_ping (mongoc_client_t *client, mongoc_client_session_t *client_session)
+{
+   bson_t ping_cmd = BSON_INITIALIZER;
+   bson_t opts = BSON_INITIALIZER;
+   bson_error_t error;
+   bool ret;
+
+   BCON_APPEND (&ping_cmd, "ping", BCON_INT32 (1));
+
+   ret = mongoc_client_session_append (client_session, &opts, &error);
+   ASSERT_OR_PRINT(ret, error);
+
+   ret = mongoc_client_command_with_opts (client,
+                                          "admin",
+                                          &ping_cmd,
+                                          NULL,
+                                          &opts,
+                                          NULL,
+                                          &error);
+   ASSERT_OR_PRINT (ret, error);
+   bson_destroy(&opts);
+   bson_destroy (&ping_cmd);
+}
 
 static void
 _test_end_sessions_many (bool pooled)
@@ -721,6 +807,7 @@ _test_end_sessions_many (bool pooled)
    for (i = 0; i < sizeof sessions / sizeof (mongoc_client_session_t *); i++) {
       sessions[i] = mongoc_client_start_session (client, NULL, &error);
       ASSERT_OR_PRINT (sessions[i], error);
+      send_ping (client, sessions[i]);
    }
 
    for (i = 0; i < sizeof sessions / sizeof (mongoc_client_session_t *); i++) {
@@ -1152,7 +1239,10 @@ check_session_returned (session_test_t *test, const bson_t *lsid)
       }
    }
 
-   if (!found) {
+   /* Server session will only be returned to the pool if it has
+    * been used. It is expected behavior for found to be false if
+    * ss->last_used_usec == SESSION_NEVER_USED */
+   if (!found && ss && ss->last_used_usec != SESSION_NEVER_USED) {
       fprintf (stderr,
                "server session %s not returned to pool\n",
                bson_as_json (lsid, NULL));
@@ -1536,6 +1626,15 @@ run_session_test_bulk_operation (void *ctx)
 }
 
 
+static void
+run_count_test (session_test_fn_t test_fn)
+{
+   /* CDRIVER-3612: mongoc_collection_estimated_document_count does not support
+    * explicit sessions */
+   _test_implicit_session_lsid (test_fn);
+}
+
+
 static void
 insert_10_docs (session_test_t *test)
 {
@@ -2161,7 +2260,11 @@ test_cursor_implicit_session (void *ctx)
    ASSERT_POOL_SIZE (topology, 0);
    ASSERT_SESSIONS_MATCH (&test->sent_lsid, &find_lsid);
 
-   /* push a new server session into the pool */
+   /* push a new server session into the pool.  server session is only pushed
+    * if it is used.  therefore mark session as used prior to
+    * destroying session by sending a ping */
+   bson_reinit (&test->sent_lsid);
+   send_ping (test->client, cs);
    mongoc_client_session_destroy (cs);
    ASSERT_POOL_SIZE (topology, 1);
    ASSERT_SESSIONS_DIFFER (&find_lsid, &topology->session_pool->lsid);
@@ -2206,7 +2309,12 @@ test_change_stream_implicit_session (void *ctx)
    ASSERT_POOL_SIZE (topology, 0);
    BSON_ASSERT (change_stream->implicit_session);
 
-   /* push a new server session into the pool */
+
+   /* push a new server session into the pool.  server session is only pushed
+    * if it is used.  therefore mark session as used prior to
+    * destroying session by sending a ping */
+   bson_reinit (&test->sent_lsid);
+   send_ping (test->client, cs);
    mongoc_client_session_destroy (cs);
    ASSERT_POOL_SIZE (topology, 1);
    ASSERT_SESSIONS_DIFFER (&aggregate_lsid, &topology->session_pool->lsid);
@@ -2563,10 +2671,13 @@ _test_session_dirty_helper (bool retry_succeeds)
       ASSERT_OR_PRINT (ret, error);
    } else {
       BSON_ASSERT (!ret);
-      ASSERT_ERROR_CONTAINS (
-         error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_SOCKET, "socket error");
+      ASSERT_ERROR_CONTAINS (error,
+                             MONGOC_ERROR_STREAM,
+                             MONGOC_ERROR_STREAM_SOCKET,
+                             "socket error");
    }
-   /* Regardless of whether the retry succeeded, the session should be marked dirty */
+   /* Regardless of whether the retry succeeded, the session should be marked
+    * dirty */
    BSON_ASSERT (session->server_session->dirty);
 
    CDL_COUNT (client->topology->session_pool, next, pooled_session_count_pre);
@@ -2687,6 +2798,10 @@ test_session_install (TestSuite *suite)
                                 "/Session/end/mock/pooled",
                                 test_mock_end_sessions_pooled,
                                 test_framework_skip_if_no_crypto);
+   TestSuite_AddMockServerTest (suite,
+                                "/Session/end/mock/disconnected",
+                                test_mock_end_sessions_server_disconnect,
+                                test_framework_skip_if_no_crypto);
    TestSuite_AddFull (suite,
                       "/Session/end/single",
                       test_end_sessions_single,
@@ -2707,14 +2822,16 @@ test_session_install (TestSuite *suite)
                       NULL,
                       NULL,
                       test_framework_skip_if_no_crypto,
-                      test_framework_skip_if_max_wire_version_less_than_6);
+                      test_framework_skip_if_max_wire_version_less_than_6,
+                      test_framework_skip_if_slow);
    TestSuite_AddFull (suite,
                       "/Session/end/many/pooled",
                       test_end_sessions_many_pooled,
                       NULL,
                       NULL,
                       test_framework_skip_if_no_crypto,
-                      test_framework_skip_if_max_wire_version_less_than_6);
+                      test_framework_skip_if_max_wire_version_less_than_6,
+                      test_framework_skip_if_slow);
    TestSuite_AddFull (suite,
                       "/Session/advance_cluster_time",
                       test_session_advance_cluster_time,
@@ -2738,7 +2855,13 @@ test_session_install (TestSuite *suite)
    add_session_test (
       suite, "/Session/read_write_cmd", test_read_write_cmd, true);
    add_session_test (suite, "/Session/db_cmd", test_db_cmd, false);
-   add_session_test (suite, "/Session/count", test_count, true);
+   TestSuite_AddFull (suite,
+                      "/Session/count",
+                      (void *) run_count_test,
+                      NULL,
+                      (void *) test_count,
+                      test_framework_skip_if_no_cluster_time,
+                      test_framework_skip_if_no_crypto);
    add_session_test (suite, "/Session/cursor", test_cursor, true);
    add_session_test (suite, "/Session/drop", test_drop, false);
    add_session_test (suite, "/Session/drop_index", test_drop_index, false);
@@ -2948,13 +3071,14 @@ test_session_install (TestSuite *suite)
                                        test_sessions_spec_cb,
                                        test_framework_skip_if_no_sessions);
 
-   TestSuite_AddFull (suite,
-                      "/Session/dirty",
-                      test_session_dirty,
-                      NULL /* dtor */,
-                      NULL /* ctx */,
-                      test_framework_skip_if_no_sessions,
-                      test_framework_skip_if_no_failpoint,
-                      /* Tests with retryable writes, requires non-standalone. */
-                      test_framework_skip_if_single);
+   TestSuite_AddFull (
+      suite,
+      "/Session/dirty",
+      test_session_dirty,
+      NULL /* dtor */,
+      NULL /* ctx */,
+      test_framework_skip_if_no_sessions,
+      test_framework_skip_if_no_failpoint,
+      /* Tests with retryable writes, requires non-standalone. */
+      test_framework_skip_if_single);
 }
