@@ -37,7 +37,7 @@ _before_test (json_test_ctx_t *ctx, const bson_t *test)
 
    if (bson_iter_init_find (&iter, ctx->config->scenario, "key_vault_data")) {
       keyvault_coll =
-         mongoc_client_get_collection (client, "admin", "datakeys");
+         mongoc_client_get_collection (client, "keyvault", "datakeys");
 
       /* Drop and recreate, inserting data. */
       ret = mongoc_collection_drop (keyvault_coll, &error);
@@ -244,9 +244,9 @@ test_bson_size_limits_and_batch_splitting (void *unused)
          client, "db", cmd, NULL /* read prefs */, NULL /* reply */, &error),
       error);
 
-   /* Drop and create the key vault collection, admin.datakeys. */
+   /* Drop and create the key vault collection, keyvault.datakeys. */
    mongoc_collection_destroy (coll);
-   coll = mongoc_client_get_collection (client, "admin", "datakeys");
+   coll = mongoc_client_get_collection (client, "keyvault", "datakeys");
    (void) mongoc_collection_drop (coll, NULL);
    datakey = get_bson_from_json_file (
       "./src/libmongoc/tests/client_side_encryption_prose/limits-key.json");
@@ -266,7 +266,7 @@ test_bson_size_limits_and_batch_splitting (void *unused)
    opts = mongoc_auto_encryption_opts_new ();
    _check_bypass (opts);
    mongoc_auto_encryption_opts_set_keyvault_namespace (
-      opts, "admin", "datakeys");
+      opts, "keyvault", "datakeys");
    mongoc_auto_encryption_opts_set_kms_providers (opts, kms_providers);
 
    ASSERT_OR_PRINT (mongoc_client_enable_auto_encryption (client, opts, &error),
@@ -443,7 +443,7 @@ test_datakey_and_double_encryption_creating_and_using (
 
    /* Expect a BSON binary with subtype 4 to be returned */
    BSON_ASSERT (keyid.value_type == BSON_TYPE_BINARY);
-   BSON_ASSERT (keyid.value.v_binary.subtype = BSON_SUBTYPE_UUID);
+   BSON_ASSERT (keyid.value.v_binary.subtype == BSON_SUBTYPE_UUID);
 
    /* Check that client captured a command_started event for the insert command
     * containing a majority writeConcern. */
@@ -452,8 +452,8 @@ test_datakey_and_double_encryption_creating_and_using (
       tmp_bson ("{'insert': 'datakeys', 'writeConcern': { 'w': 'majority' } }"),
       false));
 
-   /* Use client to run a find on admin.datakeys */
-   coll = mongoc_client_get_collection (client, "admin", "datakeys");
+   /* Use client to run a find on keyvault.datakeys */
+   coll = mongoc_client_get_collection (client, "keyvault", "datakeys");
    bson_init (&filter);
    BSON_APPEND_VALUE (&filter, "_id", &keyid);
    cursor = mongoc_collection_find_with_opts (
@@ -488,7 +488,7 @@ test_datakey_and_double_encryption_creating_and_using (
 
    /* Expect the return value to be a BSON binary subtype 6 */
    BSON_ASSERT (encrypted.value_type == BSON_TYPE_BINARY);
-   BSON_ASSERT (encrypted.value.v_binary.subtype = BSON_SUBTYPE_ENCRYPTED);
+   BSON_ASSERT (encrypted.value.v_binary.subtype == BSON_SUBTYPE_ENCRYPTED);
 
    /* Use client_encrypted to insert { _id: "<kms provider>", "value":
     * <encrypted> } into db.coll */
@@ -532,8 +532,8 @@ test_datakey_and_double_encryption_creating_and_using (
    /* Expect the return value to be a BSON binary subtype 6. Expect the value to
     * exactly match the value of encrypted. */
    BSON_ASSERT (encrypted_via_altname.value_type == BSON_TYPE_BINARY);
-   BSON_ASSERT (encrypted_via_altname.value.v_binary.subtype =
-                   BSON_SUBTYPE_ENCRYPTED);
+   BSON_ASSERT (encrypted_via_altname.value.v_binary.subtype ==
+                BSON_SUBTYPE_ENCRYPTED);
    BSON_ASSERT (encrypted_via_altname.value.v_binary.data_len ==
                 encrypted.value.v_binary.data_len);
    BSON_ASSERT (0 == memcmp (encrypted_via_altname.value.v_binary.data,
@@ -577,8 +577,8 @@ test_datakey_and_double_encryption (void *unused)
       callbacks, _datakey_and_double_encryption_command_started);
    mongoc_client_set_apm_callbacks (client, callbacks, &test_ctx);
 
-   /* Using client, drop the collections admin.datakeys and db.coll. */
-   coll = mongoc_client_get_collection (client, "admin", "datakeys");
+   /* Using client, drop the collections keyvault.datakeys and db.coll. */
+   coll = mongoc_client_get_collection (client, "keyvault", "datakeys");
    (void) mongoc_collection_drop (coll, NULL);
    mongoc_collection_destroy (coll);
    coll = mongoc_client_get_collection (client, "db", "coll");
@@ -593,7 +593,7 @@ test_datakey_and_double_encryption (void *unused)
    mongoc_auto_encryption_opts_set_kms_providers (auto_encryption_opts,
                                                   kms_providers);
    mongoc_auto_encryption_opts_set_keyvault_namespace (
-      auto_encryption_opts, "admin", "datakeys");
+      auto_encryption_opts, "keyvault", "datakeys");
    schema_map = get_bson_from_json_file (
       "./src/libmongoc/tests/client_side_encryption_prose/"
       "datakey-and-double-encryption-schemamap.json");
@@ -610,7 +610,7 @@ test_datakey_and_double_encryption (void *unused)
    mongoc_client_encryption_opts_set_kms_providers (client_encryption_opts,
                                                     kms_providers);
    mongoc_client_encryption_opts_set_keyvault_namespace (
-      client_encryption_opts, "admin", "datakeys");
+      client_encryption_opts, "keyvault", "datakeys");
    mongoc_client_encryption_opts_set_keyvault_client (client_encryption_opts,
                                                       client);
    client_encryption =
@@ -663,15 +663,15 @@ _test_key_vault (bool with_external_key_vault)
    client_external = mongoc_client_new_from_uri (external_uri);
    test_framework_set_ssl_opts (client_external);
 
-   /* Using client, drop the collections admin.datakeys and db.coll. */
+   /* Using client, drop the collections keyvault.datakeys and db.coll. */
    client = test_framework_client_new ();
    coll = mongoc_client_get_collection (client, "db", "coll");
    (void) mongoc_collection_drop (coll, NULL);
    mongoc_collection_destroy (coll);
-   coll = mongoc_client_get_collection (client, "admin", "datakeys");
+   coll = mongoc_client_get_collection (client, "keyvault", "datakeys");
    (void) mongoc_collection_drop (coll, NULL);
 
-   /* Insert the document external-key.json into ``admin.datakeys``. */
+   /* Insert the document external-key.json into ``keyvault.datakeys``. */
    wc = mongoc_write_concern_new ();
    mongoc_write_concern_set_wmajority (wc, 1000);
    mongoc_collection_set_write_concern (coll, wc);
@@ -695,7 +695,7 @@ _test_key_vault (bool with_external_key_vault)
    mongoc_auto_encryption_opts_set_kms_providers (auto_encryption_opts,
                                                   kms_providers);
    mongoc_auto_encryption_opts_set_keyvault_namespace (
-      auto_encryption_opts, "admin", "datakeys");
+      auto_encryption_opts, "keyvault", "datakeys");
    mongoc_auto_encryption_opts_set_schema_map (auto_encryption_opts,
                                                schema_map);
    if (with_external_key_vault) {
@@ -711,7 +711,7 @@ _test_key_vault (bool with_external_key_vault)
    mongoc_client_encryption_opts_set_kms_providers (client_encryption_opts,
                                                     kms_providers);
    mongoc_client_encryption_opts_set_keyvault_namespace (
-      client_encryption_opts, "admin", "datakeys");
+      client_encryption_opts, "keyvault", "datakeys");
    if (with_external_key_vault) {
       mongoc_client_encryption_opts_set_keyvault_client (client_encryption_opts,
                                                          client_external);
@@ -819,7 +819,7 @@ test_views_are_prohibited (void *unused)
    mongoc_auto_encryption_opts_set_kms_providers (auto_encryption_opts,
                                                   kms_providers);
    mongoc_auto_encryption_opts_set_keyvault_namespace (
-      auto_encryption_opts, "admin", "datakeys");
+      auto_encryption_opts, "keyvault", "datakeys");
    ASSERT_OR_PRINT (mongoc_client_enable_auto_encryption (
                        client_encrypted, auto_encryption_opts, &error),
                     error);
@@ -863,7 +863,7 @@ test_custom_endpoint (void *unused)
    mongoc_client_encryption_opts_set_kms_providers (client_encryption_opts,
                                                     kms_providers);
    mongoc_client_encryption_opts_set_keyvault_namespace (
-      client_encryption_opts, "admin", "datakeys");
+      client_encryption_opts, "keyvault", "datakeys");
    mongoc_client_encryption_opts_set_keyvault_client (client_encryption_opts,
                                                       keyvault_client);
    client_encryption =
@@ -1274,10 +1274,10 @@ _test_corpus (bool local_schema)
       ASSERT_OR_PRINT (res, error);
    }
 
-   /* Drop the collection admin.datakeys. Insert the documents
+   /* Drop the collection keyvault.datakeys. Insert the documents
     * corpus/corpus-key-local.json and corpus-key-aws.json */
    mongoc_collection_destroy (coll);
-   coll = mongoc_client_get_collection (client, "admin", "datakeys");
+   coll = mongoc_client_get_collection (client, "keyvault", "datakeys");
    (void) mongoc_collection_drop (coll, NULL);
    wc = mongoc_write_concern_new ();
    mongoc_write_concern_set_wmajority (wc, 1000);
@@ -1304,7 +1304,7 @@ _test_corpus (bool local_schema)
    mongoc_auto_encryption_opts_set_kms_providers (auto_encryption_opts,
                                                   kms_providers);
    mongoc_auto_encryption_opts_set_keyvault_namespace (
-      auto_encryption_opts, "admin", "datakeys");
+      auto_encryption_opts, "keyvault", "datakeys");
    res = mongoc_client_enable_auto_encryption (
       client_encrypted, auto_encryption_opts, &error);
    ASSERT_OR_PRINT (res, error);
@@ -1314,7 +1314,7 @@ _test_corpus (bool local_schema)
    mongoc_client_encryption_opts_set_kms_providers (client_encryption_opts,
                                                     kms_providers);
    mongoc_client_encryption_opts_set_keyvault_namespace (
-      client_encryption_opts, "admin", "datakeys");
+      client_encryption_opts, "keyvault", "datakeys");
    mongoc_client_encryption_opts_set_keyvault_client (client_encryption_opts,
                                                       client);
    client_encryption =
@@ -1636,8 +1636,7 @@ test_invalid_single_and_pool_mismatches (void *unused)
    mongoc_auto_encryption_opts_destroy (opts);
 }
 
-static void *
-_worker_thread (void *client_ptr)
+static BSON_THREAD_FUN (_worker_thread, client_ptr)
 {
    mongoc_client_t *client_encrypted;
    mongoc_collection_t *coll;
@@ -1664,7 +1663,7 @@ _worker_thread (void *client_ptr)
    mongoc_collection_destroy (coll);
    bson_destroy (&filter);
    bson_destroy (to_insert);
-   return NULL;
+   BSON_THREAD_RETURN;
 }
 
 static void
@@ -1728,14 +1727,14 @@ _test_multi_threaded (bool external_key_vault)
    client1 = mongoc_client_pool_pop (pool);
    client2 = mongoc_client_pool_pop (pool);
 
-   r = bson_thread_create (threads, _worker_thread, client1);
+   r = COMMON_PREFIX (thread_create) (threads, _worker_thread, client1);
    BSON_ASSERT (r == 0);
 
-   r = bson_thread_create (threads + 1, _worker_thread, client2);
+   r = COMMON_PREFIX (thread_create) (threads + 1, _worker_thread, client2);
    BSON_ASSERT (r == 0);
 
    for (i = 0; i < 2; i++) {
-      r = bson_thread_join (threads[i]);
+      r = COMMON_PREFIX (thread_join) (threads[i]);
       BSON_ASSERT (r == 0);
    }
 
@@ -1780,7 +1779,7 @@ test_malformed_explicit (void *unused)
    mongoc_client_encryption_opts_set_kms_providers (client_encryption_opts,
                                                     kms_providers);
    mongoc_client_encryption_opts_set_keyvault_namespace (
-      client_encryption_opts, "admin", "datakeys");
+      client_encryption_opts, "keyvault", "datakeys");
    mongoc_client_encryption_opts_set_keyvault_client (client_encryption_opts,
                                                       client);
    client_encryption =
@@ -1801,6 +1800,145 @@ test_malformed_explicit (void *unused)
    mongoc_client_destroy (client);
 }
 
+static void
+_check_mongocryptd_not_spawned (void)
+{
+   mongoc_client_t *client;
+   bson_t *cmd;
+   bson_error_t error;
+   bool ret;
+
+   client = mongoc_client_new (
+      "mongodb://localhost:27021/db?serverSelectionTimeoutMS=1000");
+   cmd = BCON_NEW ("ismaster", BCON_INT32 (1));
+   ret = mongoc_client_command_simple (
+      client, "keyvault", cmd, NULL /* read prefs */, NULL /* reply */, &error);
+   BSON_ASSERT (!ret);
+   ASSERT_ERROR_CONTAINS (error,
+                          MONGOC_ERROR_SERVER_SELECTION,
+                          MONGOC_ERROR_SERVER_SELECTION_FAILURE,
+                          "No suitable servers");
+   mongoc_client_destroy (client);
+   bson_destroy (cmd);
+}
+
+static void
+test_bypass_spawning_via_mongocryptdBypassSpawn (void *unused)
+{
+   mongoc_client_t *client_encrypted;
+   mongoc_auto_encryption_opts_t *auto_encryption_opts;
+   bson_t *kms_providers;
+   bson_t *doc_to_insert;
+   bson_t *extra;
+   bson_t *schema_map;
+   bson_t *schema;
+   bool ret;
+   bson_error_t error;
+   mongoc_collection_t *coll;
+
+   auto_encryption_opts = mongoc_auto_encryption_opts_new ();
+   kms_providers = _make_kms_providers (false /* aws */, true /* local */);
+   mongoc_auto_encryption_opts_set_kms_providers (auto_encryption_opts,
+                                                  kms_providers);
+   mongoc_auto_encryption_opts_set_keyvault_namespace (
+      auto_encryption_opts, "keyvault", "datakeys");
+   schema = get_bson_from_json_file ("./src/libmongoc/tests/"
+                                     "client_side_encryption_prose/external/"
+                                     "external-schema.json");
+   schema_map = BCON_NEW ("db.coll", BCON_DOCUMENT (schema));
+
+   /* Create a MongoClient with encryption enabled */
+   client_encrypted = test_framework_client_new ();
+   extra =
+      BCON_NEW ("mongocryptdBypassSpawn",
+                BCON_BOOL (true),
+                "mongocryptdSpawnArgs",
+                "[",
+                "--pidfilepath=bypass-spawning-mongocryptd.pid",
+                "--port=27021",
+                "]",
+                "mongocryptdURI",
+                "mongodb://localhost:27021/?serverSelectionTimeoutMS=1000");
+   mongoc_auto_encryption_opts_set_extra (auto_encryption_opts, extra);
+   mongoc_auto_encryption_opts_set_schema_map (auto_encryption_opts,
+                                               schema_map);
+   bson_destroy (extra);
+   ret = mongoc_client_enable_auto_encryption (
+      client_encrypted, auto_encryption_opts, &error);
+   ASSERT_OR_PRINT (ret, error);
+
+   /* Insert { 'encrypt': 'test' }. Should fail with a server selection error.
+    */
+   coll = mongoc_client_get_collection (client_encrypted, "db", "coll");
+   doc_to_insert = BCON_NEW ("encrypt", "test");
+   ret = mongoc_collection_insert_one (
+      coll, doc_to_insert, NULL /* opts */, NULL /* reply */, &error);
+   BSON_ASSERT (!ret);
+   ASSERT_ERROR_CONTAINS (error,
+                          MONGOC_ERROR_SERVER_SELECTION,
+                          MONGOC_ERROR_SERVER_SELECTION_FAILURE,
+                          "mongocryptd error: No suitable servers found");
+
+   _check_mongocryptd_not_spawned ();
+   bson_destroy (schema_map);
+   bson_destroy (schema);
+   mongoc_collection_destroy (coll);
+   mongoc_client_destroy (client_encrypted);
+   bson_destroy (doc_to_insert);
+   mongoc_auto_encryption_opts_destroy (auto_encryption_opts);
+   bson_destroy (kms_providers);
+}
+
+static void
+test_bypass_spawning_via_bypassAutoEncryption (void *unused)
+{
+   mongoc_client_t *client_encrypted;
+   mongoc_auto_encryption_opts_t *auto_encryption_opts;
+   bson_t *kms_providers;
+   bson_t *doc_to_insert;
+   bson_t *extra;
+   bool ret;
+   bson_error_t error;
+   mongoc_collection_t *coll;
+
+   auto_encryption_opts = mongoc_auto_encryption_opts_new ();
+   kms_providers = _make_kms_providers (false /* aws */, true /* local */);
+   mongoc_auto_encryption_opts_set_kms_providers (auto_encryption_opts,
+                                                  kms_providers);
+   mongoc_auto_encryption_opts_set_keyvault_namespace (
+      auto_encryption_opts, "keyvault", "datakeys");
+   mongoc_auto_encryption_opts_set_bypass_auto_encryption (auto_encryption_opts,
+                                                           true);
+
+   /* Create a MongoClient with encryption enabled */
+   client_encrypted = test_framework_client_new ();
+   extra = BCON_NEW ("mongocryptdSpawnArgs",
+                     "[",
+                     "--pidfilepath=bypass-spawning-mongocryptd.pid",
+                     "--port=27021",
+                     "]");
+   mongoc_auto_encryption_opts_set_extra (auto_encryption_opts, extra);
+   bson_destroy (extra);
+   ret = mongoc_client_enable_auto_encryption (
+      client_encrypted, auto_encryption_opts, &error);
+   ASSERT_OR_PRINT (ret, error);
+
+   /* Insert { 'encrypt': 'test' }. Should succeed. */
+   coll = mongoc_client_get_collection (client_encrypted, "db", "coll");
+   doc_to_insert = BCON_NEW ("unencrypted", "test");
+   ret = mongoc_collection_insert_one (
+      coll, doc_to_insert, NULL /* opts */, NULL /* reply */, &error);
+   ASSERT_OR_PRINT (ret, error);
+
+   _check_mongocryptd_not_spawned ();
+
+   mongoc_collection_destroy (coll);
+   mongoc_client_destroy (client_encrypted);
+   bson_destroy (doc_to_insert);
+   mongoc_auto_encryption_opts_destroy (auto_encryption_opts);
+   bson_destroy (kms_providers);
+}
+
 void
 test_client_side_encryption_install (TestSuite *suite)
 {
@@ -1861,6 +1999,22 @@ test_client_side_encryption_install (TestSuite *suite)
                       test_framework_skip_if_no_client_side_encryption,
                       test_framework_skip_if_max_wire_version_less_than_8,
                       test_framework_skip_if_offline /* requires AWS */);
+   TestSuite_AddFull (suite,
+                      "/client_side_encryption/bypass_spawning_mongocryptd/"
+                      "mongocryptdBypassSpawn",
+                      test_bypass_spawning_via_mongocryptdBypassSpawn,
+                      NULL,
+                      NULL,
+                      test_framework_skip_if_no_client_side_encryption,
+                      test_framework_skip_if_max_wire_version_less_than_8);
+   TestSuite_AddFull (suite,
+                      "/client_side_encryption/bypass_spawning_mongocryptd/"
+                      "bypassAutoEncryption",
+                      test_bypass_spawning_via_bypassAutoEncryption,
+                      NULL,
+                      NULL,
+                      test_framework_skip_if_no_client_side_encryption,
+                      test_framework_skip_if_max_wire_version_less_than_8);
    /* Other, C driver specific, tests. */
    TestSuite_AddFull (suite,
                       "/client_side_encryption/single_and_pool_mismatches",
@@ -1883,4 +2037,4 @@ test_client_side_encryption_install (TestSuite *suite)
                       NULL,
                       test_framework_skip_if_no_client_side_encryption,
                       test_framework_skip_if_max_wire_version_less_than_8);
-}
\ No newline at end of file
+}
