@@ -40,7 +40,15 @@ test_all_spec_tests (TestSuite *suite)
    install_json_test_suite_with_check (suite,
                                        resolved,
                                        &test_crud_cb,
-                                       test_framework_skip_if_no_sessions);
+                                       test_framework_skip_if_no_crypto,
+                                       TestSuite_CheckLive);
+
+   /* Read/write concern spec tests use the same format. */
+   test_framework_resolve_path (JSON_DIR "/read_write_concern/operation",
+                                resolved);
+
+   install_json_test_suite_with_check (
+      suite, resolved, &test_crud_cb, TestSuite_CheckLive);
 }
 
 void
