@@ -785,6 +785,10 @@ test_handshake_platform_config ()
       _get_bit (config_str, MONGOC_MD_FLAG_ENABLE_CLIENT_SIDE_ENCRYPTION));
 #endif
 
+#ifdef MONGOC_ENABLE_MONGODB_AWS_AUTH
+   BSON_ASSERT (_get_bit (config_str, MONGOC_MD_FLAG_ENABLE_MONGODB_AWS_AUTH));
+#endif
+
    /* any excess bits should all be zero. */
    for (i = LAST_MONGOC_MD_FLAG; i < total_bits; i++) {
       BSON_ASSERT (!_get_bit (config_str, i));
@@ -793,8 +797,7 @@ test_handshake_platform_config ()
 }
 
 /* Called by a single thread in test_mongoc_handshake_race_condition */
-static void *
-handshake_append_worker (void *data)
+static BSON_THREAD_FUN (handshake_append_worker, data)
 {
    const char *driver_name = "php driver";
    const char *driver_version = "version abc";
@@ -802,7 +805,7 @@ handshake_append_worker (void *data)
 
    mongoc_handshake_data_append (driver_name, driver_version, platform);
 
-   return NULL;
+   BSON_THREAD_RETURN;
 }
 
 /* Run 1000 iterations of mongoc_handshake_data_append() using 4 threads */
@@ -816,12 +819,12 @@ test_mongoc_handshake_race_condition (void)
       _reset_handshake ();
 
       for (j = 0; j < 4; ++j) {
-         BSON_ASSERT (
-            !bson_thread_create (&threads[j], &handshake_append_worker, NULL));
+         BSON_ASSERT (!COMMON_PREFIX (thread_create) (
+            &threads[j], &handshake_append_worker, NULL));
       }
 
       for (j = 0; j < 4; ++j) {
-         bson_thread_join (threads[j]);
+         COMMON_PREFIX (thread_join) (threads[j]);
       }
    }
 
