@@ -1,6 +1,6 @@
-#include "mongoc.h"
-#include "mongoc-read-concern-private.h"
-#include "mongoc-util-private.h"
+#include "mongoc/mongoc.h"
+#include "mongoc/mongoc-read-concern-private.h"
+#include "mongoc/mongoc-util-private.h"
 
 #include "TestSuite.h"
 #include "test-conveniences.h"
