@@ -35,8 +35,8 @@ check_json_sdam_events (const bson_t *events, const bson_t *expectations)
 
    if (expected_keys != actual_keys) {
       test_error ("SDAM test failed expectations:\n\n"
-                  "%s\n\n"
-                  "events:\n%s\n\n"
+                  "expected\n%s\n\n"
+                  "actual:\n%s\n\n"
                   "expected %" PRIu32 " events, got %" PRIu32,
                   bson_as_canonical_extended_json (expectations, NULL),
                   bson_as_canonical_extended_json (events, NULL),
@@ -46,8 +46,8 @@ check_json_sdam_events (const bson_t *events, const bson_t *expectations)
 
    if (!match_bson_with_ctx (events, expectations, &match_ctx)) {
       test_error ("SDAM test failed expectations:\n\n"
-                  "%s\n\n"
-                  "events:\n%s\n\n%s",
+                  "expected\n%s\n\n"
+                  "actual:\n%s\n\n%s",
                   bson_as_canonical_extended_json (expectations, NULL),
                   bson_as_canonical_extended_json (events, NULL),
                   match_ctx.errmsg);
@@ -313,12 +313,16 @@ server_heartbeat_started (const mongoc_apm_server_heartbeat_started_t *event)
 
    ctx = (context_t *) mongoc_apm_server_heartbeat_started_get_context (event);
    host = mongoc_apm_server_heartbeat_started_get_host (event);
-   context_append (ctx,
-                   BCON_NEW ("heartbeat_started_event",
-                             "{",
-                             "host",
-                             BCON_UTF8 (host->host_and_port),
-                             "}"));
+   context_append (
+      ctx,
+      BCON_NEW (
+         "heartbeat_started_event",
+         "{",
+         "host",
+         BCON_UTF8 (host->host_and_port),
+         "awaited",
+         BCON_BOOL (mongoc_apm_server_heartbeat_started_get_awaited (event)),
+         "}"));
 }
 
 static void
@@ -332,12 +336,16 @@ server_heartbeat_succeeded (
    ctx =
       (context_t *) mongoc_apm_server_heartbeat_succeeded_get_context (event);
    host = mongoc_apm_server_heartbeat_succeeded_get_host (event);
-   context_append (ctx,
-                   BCON_NEW ("heartbeat_succeeded_event",
-                             "{",
-                             "host",
-                             BCON_UTF8 (host->host_and_port),
-                             "}"));
+   context_append (
+      ctx,
+      BCON_NEW (
+         "heartbeat_succeeded_event",
+         "{",
+         "host",
+         BCON_UTF8 (host->host_and_port),
+         "awaited",
+         BCON_BOOL (mongoc_apm_server_heartbeat_succeeded_get_awaited (event)),
+         "}"));
 
    duration = mongoc_apm_server_heartbeat_succeeded_get_duration (event);
    _mongoc_array_append_val (&ctx->heartbeat_succeeded_durations, duration);
@@ -352,12 +360,16 @@ server_heartbeat_failed (const mongoc_apm_server_heartbeat_failed_t *event)
 
    ctx = (context_t *) mongoc_apm_server_heartbeat_failed_get_context (event);
    host = mongoc_apm_server_heartbeat_failed_get_host (event);
-   context_append (ctx,
-                   BCON_NEW ("heartbeat_failed_event",
-                             "{",
-                             "host",
-                             BCON_UTF8 (host->host_and_port),
-                             "}"));
+   context_append (
+      ctx,
+      BCON_NEW (
+         "heartbeat_failed_event",
+         "{",
+         "host",
+         BCON_UTF8 (host->host_and_port),
+         "awaited",
+         BCON_BOOL (mongoc_apm_server_heartbeat_failed_get_awaited (event)),
+         "}"));
 
    duration = mongoc_apm_server_heartbeat_failed_get_duration (event);
    _mongoc_array_append_val (&ctx->heartbeat_failed_durations, duration);
@@ -479,9 +491,9 @@ test_sdam_monitoring_cb (bson_t *test)
       bson_iter_bson (&phase_iter, &phase);
 
       if (first_phase) {
-         /* Force the topology opening and server opening events. This test doesn't
-          * exercise this code path naturally, see below in _test_topology_events
-          * for a non-hacky test of this event */
+         /* Force the topology opening and server opening events. This test
+          * doesn't exercise this code path naturally, see below in
+          * _test_topology_events for a non-hacky test of this event */
          _mongoc_topology_description_monitor_opening (&topology->description);
          first_phase = false;
       } else {
@@ -489,8 +501,7 @@ test_sdam_monitoring_cb (bson_t *test)
          bson_reinit (&context.events);
          context.n_events = 0;
       }
-      process_sdam_test_ismaster_responses (&phase,
-                                            &client->topology->description);
+      process_sdam_test_ismaster_responses (&phase, client->topology);
 
       /* parse out "outcome" and validate */
       BSON_ASSERT (bson_iter_init_find (&phase_field_iter, &phase, "outcome"));
@@ -682,6 +693,10 @@ _test_heartbeat_events (bool pooled, bool succeeded)
    uri = mongoc_uri_copy (mock_server_get_uri (server));
    mongoc_uri_set_option_as_int32 (uri, "serverSelectionTimeoutMS", 400);
 
+   /* The start time should be before scanning is started (before the call to
+    * mongoc_client_pool_pop for pooled) */
+   start = bson_get_monotonic_time ();
+
    if (pooled) {
       pool = mongoc_client_pool_new (uri);
       pool_set_heartbeat_event_callbacks (pool, &context);
@@ -691,8 +706,6 @@ _test_heartbeat_events (bool pooled, bool succeeded)
       client_set_heartbeat_event_callbacks (client, &context);
    }
 
-   start = bson_get_monotonic_time ();
-
    /* trigger "ismaster" handshake */
    future = future_client_command_simple (
       client, "admin", tmp_bson ("{'foo': 1}"), NULL, NULL, &error);
@@ -747,15 +760,16 @@ _test_heartbeat_events (bool pooled, bool succeeded)
    if (succeeded) {
       durations = &context.heartbeat_succeeded_durations;
       expected_json = bson_strdup_printf (
-         "{'0': {'heartbeat_started_event': {'host': '%s'}},"
-         " '1': {'heartbeat_succeeded_event': {'host': '%s'}}}",
+         "{'0': {'heartbeat_started_event': {'host': '%s', 'awaited': false}},"
+         " '1': {'heartbeat_succeeded_event': {'host': '%s', 'awaited': "
+         "false}}}",
          mock_server_get_host_and_port (server),
          mock_server_get_host_and_port (server));
    } else {
       durations = &context.heartbeat_failed_durations;
       expected_json = bson_strdup_printf (
-         "{'0': {'heartbeat_started_event': {'host': '%s'}},"
-         " '1': {'heartbeat_failed_event': {'host': '%s'}}}",
+         "{'0': {'heartbeat_started_event': {'host': '%s', 'awaited': false}},"
+         " '1': {'heartbeat_failed_event': {'host': '%s', 'awaited': false}}}",
          mock_server_get_host_and_port (server),
          mock_server_get_host_and_port (server));
    }
@@ -815,8 +829,11 @@ _test_heartbeat_fails_dns (bool pooled)
    size_t i;
 
    context_init (&context);
+   /* Track time before scanning starts (which is triggered by popping the first
+    * client for a client pool). */
+   start = bson_get_monotonic_time ();
    uri = mongoc_uri_new (
-      "mongodb://doesntexist.foobar/?serverSelectionTimeoutMS=1000");
+      "mongodb://doesntexist.foobar/?serverSelectionTimeoutMS=3000");
    if (pooled) {
       pool = mongoc_client_pool_new (uri);
       pool_set_heartbeat_event_callbacks (pool, &context);
@@ -826,17 +843,17 @@ _test_heartbeat_fails_dns (bool pooled)
       client_set_heartbeat_event_callbacks (client, &context);
    }
 
-   start = bson_get_monotonic_time ();
-
    /* trigger "ismaster" handshake */
    r = mongoc_client_command_simple (
       client, "admin", tmp_bson ("{'foo': 1}"), NULL, NULL, &error);
 
+   /* This should result in either a DNS failure or connection failure depending
+    * on the network. We assert the domain/code but not the message string. */
    ASSERT (!r);
    ASSERT_ERROR_CONTAINS (error,
                           MONGOC_ERROR_SERVER_SELECTION,
                           MONGOC_ERROR_SERVER_SELECTION_FAILURE,
-                          "Failed to resolve");
+                          "");
 
    duration = bson_get_monotonic_time () - start;
 
@@ -872,6 +889,110 @@ test_heartbeat_fails_dns_pooled (void *ctx)
    _test_heartbeat_fails_dns (true);
 }
 
+typedef struct {
+   uint32_t num_server_description_changed_events;
+   uint32_t num_topology_description_changed_events;
+} duplicates_counter_t;
+
+void
+duplicates_server_changed (const mongoc_apm_server_changed_t *event)
+{
+   duplicates_counter_t *counters;
+
+   counters = mongoc_apm_server_changed_get_context (event);
+   counters->num_server_description_changed_events++;
+}
+
+void
+duplicates_topology_changed (const mongoc_apm_topology_changed_t *event)
+{
+   duplicates_counter_t *counters;
+
+   counters = mongoc_apm_topology_changed_get_context (event);
+   counters->num_topology_description_changed_events++;
+}
+
+/* Test that duplicate ismaster responses do not trigger two server
+ * description changed events or topology changed events */
+static void
+test_no_duplicates (void)
+{
+   mock_server_t *server;
+   mongoc_client_pool_t *pool;
+   mongoc_client_t *client;
+   request_t *request;
+   bson_error_t error;
+   future_t *future;
+   mongoc_uri_t *uri;
+   char *first_ismaster_response =
+      "{'ok': 1.0, 'ismaster': true, 'minWireVersion': 0, 'maxWireVersion': 9}";
+   char *second_ismaster_response = "{'ok': 1.0, 'ismaster': true, "
+                                    "'minWireVersion': 0, 'maxWireVersion': 9, "
+                                    "'lastWrite': {'lastWriteDate': {'$date': "
+                                    "{'$numberLong': '123'}}, 'opTime': 2}}";
+   mongoc_apm_callbacks_t *callbacks;
+   duplicates_counter_t duplicates_counter = {0};
+   mongoc_server_description_t *sd;
+
+   server = mock_server_new ();
+   mock_server_run (server);
+
+   callbacks = mongoc_apm_callbacks_new ();
+   /* Set a high heartbeat frequency ms to prevent periodic background scanning
+    * from interfering. */
+   uri = mongoc_uri_copy (mock_server_get_uri (server));
+   mongoc_uri_set_option_as_int32 (uri, MONGOC_URI_HEARTBEATFREQUENCYMS, 99999);
+   pool = mongoc_client_pool_new (uri);
+   mongoc_apm_set_server_changed_cb (callbacks, duplicates_server_changed);
+   mongoc_apm_set_topology_changed_cb (callbacks, duplicates_topology_changed);
+   mongoc_client_pool_set_apm_callbacks (pool, callbacks, &duplicates_counter);
+   client = mongoc_client_pool_pop (pool);
+
+   /* Topology scanning thread starts, and sends an ismaster. */
+   request = mock_server_receives_ismaster (server);
+   mock_server_replies_simple (request, first_ismaster_response);
+   request_destroy (request);
+
+   /* Perform a ping, which creates a new connection, which performs the
+    * ismaster handshake before sending the ping command. */
+   future = future_client_command_simple (client,
+                                          "admin",
+                                          tmp_bson ("{'ping': 1}"),
+                                          NULL /* read prefs */,
+                                          NULL /* reply */,
+                                          &error);
+   request = mock_server_receives_ismaster (server);
+   mock_server_replies_simple (request, second_ismaster_response);
+   request_destroy (request);
+   request = mock_server_receives_msg (
+      server, MONGOC_QUERY_NONE, tmp_bson ("{'ping': 1}"));
+   mock_server_replies_ok_and_destroys (request);
+   ASSERT_OR_PRINT (future_get_bool (future), error);
+   future_destroy (future);
+
+   ASSERT_CMPINT (
+      duplicates_counter.num_server_description_changed_events, ==, 1);
+   /* There should be two topology changed events. One for the initial topology
+    * (where the server is set to Unknown), and one for the first ismaster (but
+    * not the second) */
+   ASSERT_CMPINT (
+      duplicates_counter.num_topology_description_changed_events, ==, 2);
+
+   /* Even though no topology description changed event was emitted, the newly
+    * created server description should still overwrite the old one in the
+    * topology description. It differs in that it has the 'lastWrite' field,
+    * which does not have an effect in equality comparison. */
+   sd = mongoc_client_get_server_description (client, 1);
+   BSON_ASSERT (bson_has_field (&sd->last_is_master, "lastWrite"));
+   mongoc_server_description_destroy (sd);
+
+   mongoc_uri_destroy (uri);
+   mongoc_apm_callbacks_destroy (callbacks);
+   mock_server_destroy (server);
+   mongoc_client_pool_push (pool, client);
+   mongoc_client_pool_destroy (pool);
+}
+
 void
 test_sdam_monitoring_install (TestSuite *suite)
 {
@@ -900,10 +1021,12 @@ test_sdam_monitoring_install (TestSuite *suite)
       suite,
       "/server_discovery_and_monitoring/monitoring/heartbeat/pooled/succeeded",
       test_heartbeat_events_pooled_succeeded);
-   TestSuite_AddMockServerTest (
+   _TestSuite_AddMockServerTest (
       suite,
       "/server_discovery_and_monitoring/monitoring/heartbeat/pooled/failed",
-      test_heartbeat_events_pooled_failed);
+      test_heartbeat_events_pooled_failed,
+      test_framework_skip_if_time_sensitive,
+      NULL);
    TestSuite_AddFull (
       suite,
       "/server_discovery_and_monitoring/monitoring/heartbeat/single/dns",
@@ -918,4 +1041,10 @@ test_sdam_monitoring_install (TestSuite *suite)
       NULL,
       NULL,
       test_framework_skip_if_offline);
+   TestSuite_AddMockServerTest (
+      suite,
+      "/server_discovery_and_monitoring/monitoring/no_duplicates",
+      test_no_duplicates,
+      NULL,
+      NULL);
 }
