@@ -28,8 +28,7 @@ typedef struct {
 } socket_test_data_t;
 
 
-static void *
-socket_test_server (void *data_)
+static BSON_THREAD_FUN (socket_test_server, data_)
 {
    socket_test_data_t *data = (socket_test_data_t *) data_;
    struct sockaddr_in server_addr = {0};
@@ -97,12 +96,11 @@ socket_test_server (void *data_)
 
    mongoc_socket_destroy (listen_sock);
 
-   return NULL;
+   BSON_THREAD_RETURN;
 }
 
 
-static void *
-socket_test_client (void *data_)
+static BSON_THREAD_FUN (socket_test_client, data_)
 {
    socket_test_data_t *data = (socket_test_data_t *) data_;
    int64_t start;
@@ -173,12 +171,11 @@ socket_test_client (void *data_)
 
    mongoc_stream_destroy (stream);
 
-   return NULL;
+   BSON_THREAD_RETURN;
 }
 
 
-static void *
-sendv_test_server (void *data_)
+static BSON_THREAD_FUN (sendv_test_server, data_)
 {
    socket_test_data_t *data = (socket_test_data_t *) data_;
    struct sockaddr_in server_addr = {0};
@@ -264,12 +261,11 @@ sendv_test_server (void *data_)
    mongoc_stream_destroy (stream);
    mongoc_socket_destroy (listen_sock);
 
-   return NULL;
+   BSON_THREAD_RETURN;
 }
 
 
-static void *
-sendv_test_client (void *data_)
+static BSON_THREAD_FUN (sendv_test_client, data_)
 {
    socket_test_data_t *data = (socket_test_data_t *) data_;
    mongoc_socket_t *conn_sock;
@@ -333,7 +329,7 @@ sendv_test_client (void *data_)
    mongoc_stream_destroy (stream);
    bson_free (buf);
 
-   return NULL;
+   BSON_THREAD_RETURN;
 }
 
 
@@ -348,14 +344,14 @@ _test_mongoc_socket_check_closed (int32_t server_sleep_ms)
    mongoc_cond_init (&data.cond);
    data.server_sleep_ms = server_sleep_ms;
 
-   r = bson_thread_create (threads, &socket_test_server, &data);
+   r = COMMON_PREFIX (thread_create) (threads, &socket_test_server, &data);
    BSON_ASSERT (r == 0);
 
-   r = bson_thread_create (threads + 1, &socket_test_client, &data);
+   r = COMMON_PREFIX (thread_create) (threads + 1, &socket_test_client, &data);
    BSON_ASSERT (r == 0);
 
    for (i = 0; i < 2; i++) {
-      r = bson_thread_join (threads[i]);
+      r = COMMON_PREFIX (thread_join) (threads[i]);
       BSON_ASSERT (r == 0);
    }
 
@@ -388,14 +384,14 @@ test_mongoc_socket_sendv (void *ctx)
    bson_mutex_init (&data.cond_mutex);
    mongoc_cond_init (&data.cond);
 
-   r = bson_thread_create (threads, &sendv_test_server, &data);
+   r = COMMON_PREFIX (thread_create) (threads, &sendv_test_server, &data);
    BSON_ASSERT (r == 0);
 
-   r = bson_thread_create (threads + 1, &sendv_test_client, &data);
+   r = COMMON_PREFIX (thread_create) (threads + 1, &sendv_test_client, &data);
    BSON_ASSERT (r == 0);
 
    for (i = 0; i < 2; i++) {
-      r = bson_thread_join (threads[i]);
+      r = COMMON_PREFIX (thread_join) (threads[i]);
       BSON_ASSERT (r == 0);
    }
 
