@@ -4,6 +4,7 @@
 
 #include "mongoc/mongoc-client-private.h"
 #include "mongoc/mongoc-util-private.h"
+#include "mongoc/mongoc-topology-background-monitoring-private.h"
 #include "TestSuite.h"
 
 #include "test-libmongoc.h"
@@ -15,6 +16,123 @@
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "topology-test"
 
+typedef struct {
+   int n_started;
+   int n_succeeded;
+   int n_failed;
+   int n_unknowns;
+   bson_mutex_t mutex;
+} checks_t;
+
+static void
+checks_init (checks_t *checks)
+{
+   memset (checks, 0, sizeof (*checks));
+   bson_mutex_init (&checks->mutex);
+}
+
+static void
+checks_cleanup (checks_t *checks)
+{
+   bson_mutex_destroy (&checks->mutex);
+}
+
+static bool
+checks_cmp (checks_t *checks, const char *metric, char cmp, int expected)
+{
+   int actual = 0;
+
+   bson_mutex_lock (&checks->mutex);
+   if (0 == strcmp (metric, "n_started")) {
+      actual = checks->n_started;
+   } else if (0 == strcmp (metric, "n_succeeded")) {
+      actual = checks->n_succeeded;
+   } else if (0 == strcmp (metric, "n_failed")) {
+      actual = checks->n_failed;
+   } else if (0 == strcmp (metric, "n_unknowns")) {
+      actual = checks->n_unknowns;
+   } else {
+      test_error ("unknown metric: %s", metric);
+   }
+
+   bson_mutex_unlock (&checks->mutex);
+
+   if (cmp == '=') {
+      return actual == expected;
+   } else if (cmp == '>') {
+      return actual > expected;
+   } else if (cmp == '<') {
+      return actual < expected;
+   } else {
+      test_error ("unknown comparison: %c", cmp);
+   }
+   return false;
+}
+
+static void
+check_started (const mongoc_apm_server_heartbeat_started_t *event)
+{
+   checks_t *c;
+
+   c = (checks_t *) mongoc_apm_server_heartbeat_started_get_context (event);
+   bson_mutex_lock (&c->mutex);
+   c->n_started++;
+   bson_mutex_unlock (&c->mutex);
+}
+
+
+static void
+check_succeeded (const mongoc_apm_server_heartbeat_succeeded_t *event)
+{
+   checks_t *c;
+
+   c = (checks_t *) mongoc_apm_server_heartbeat_succeeded_get_context (event);
+   bson_mutex_lock (&c->mutex);
+   c->n_succeeded++;
+   bson_mutex_unlock (&c->mutex);
+}
+
+
+static void
+check_failed (const mongoc_apm_server_heartbeat_failed_t *event)
+{
+   checks_t *c;
+
+   c = (checks_t *) mongoc_apm_server_heartbeat_failed_get_context (event);
+   bson_mutex_lock (&c->mutex);
+   c->n_failed++;
+   bson_mutex_unlock (&c->mutex);
+}
+
+static void
+server_changed_callback (const mongoc_apm_server_changed_t *event)
+{
+   checks_t *c;
+   const mongoc_server_description_t *sd;
+
+   c = (checks_t *) mongoc_apm_server_changed_get_context (event);
+   bson_mutex_lock (&c->mutex);
+   sd = mongoc_apm_server_changed_get_new_description (event);
+   if (sd->type == MONGOC_SERVER_UNKNOWN) {
+      c->n_unknowns++;
+   }
+   bson_mutex_unlock (&c->mutex);
+}
+
+
+static mongoc_apm_callbacks_t *
+heartbeat_callbacks (void)
+{
+   mongoc_apm_callbacks_t *callbacks;
+
+   callbacks = mongoc_apm_callbacks_new ();
+   mongoc_apm_set_server_heartbeat_started_cb (callbacks, check_started);
+   mongoc_apm_set_server_heartbeat_succeeded_cb (callbacks, check_succeeded);
+   mongoc_apm_set_server_heartbeat_failed_cb (callbacks, check_failed);
+   mongoc_apm_set_server_changed_cb (callbacks, server_changed_callback);
+
+   return callbacks;
+}
 
 static void
 test_topology_client_creation (void)
@@ -99,25 +217,35 @@ test_topology_thread_start_stop (void)
    topology = _mongoc_client_pool_get_topology (pool);
 
    /* Test starting up the scanner */
-   ASSERT (_mongoc_topology_start_background_scanner (topology));
+   bson_mutex_lock (&topology->mutex);
+   _mongoc_topology_background_monitoring_start (topology);
+   bson_mutex_unlock (&topology->mutex);
    assert_topology_state (topology, MONGOC_TOPOLOGY_SCANNER_BG_RUNNING);
 
    /* Test that starting the topology while it is already
       running is ok to do. */
-   ASSERT (_mongoc_topology_start_background_scanner (topology));
+   bson_mutex_lock (&topology->mutex);
+   _mongoc_topology_background_monitoring_start (topology);
+   bson_mutex_unlock (&topology->mutex);
    assert_topology_state (topology, MONGOC_TOPOLOGY_SCANNER_BG_RUNNING);
 
    /* Test that we can stop the topology */
-   _mongoc_topology_background_thread_stop (topology);
+   bson_mutex_lock (&topology->mutex);
+   _mongoc_topology_background_monitoring_stop (topology);
+   bson_mutex_unlock (&topology->mutex);
    assert_topology_state (topology, MONGOC_TOPOLOGY_SCANNER_OFF);
 
    /* Test that stopping the topology when it is already
       stopped is ok to do. */
-   _mongoc_topology_background_thread_stop (topology);
+   bson_mutex_lock (&topology->mutex);
+   _mongoc_topology_background_monitoring_stop (topology);
+   bson_mutex_unlock (&topology->mutex);
    assert_topology_state (topology, MONGOC_TOPOLOGY_SCANNER_OFF);
 
    /* Test that we can start the topology again after stopping it */
-   ASSERT (_mongoc_topology_start_background_scanner (topology));
+   bson_mutex_lock (&topology->mutex);
+   _mongoc_topology_background_monitoring_start (topology);
+   bson_mutex_unlock (&topology->mutex);
    assert_topology_state (topology, MONGOC_TOPOLOGY_SCANNER_BG_RUNNING);
 
    mongoc_client_pool_destroy (pool);
@@ -339,17 +467,27 @@ _test_topology_invalidate_server (bool pooled)
    uint32_t fake_id = 42;
    uint32_t id;
    mongoc_server_stream_t *server_stream;
+   checks_t checks;
+   int server_count;
+   mongoc_apm_callbacks_t *callbacks;
 
+   checks_init (&checks);
    uri = test_framework_get_uri ();
    /* no auto heartbeat */
    mongoc_uri_set_option_as_int32 (uri, "heartbeatFrequencyMS", INT32_MAX);
-   mongoc_uri_set_option_as_int32 (uri, "connectTimeoutMS", 2000);
+   mongoc_uri_set_option_as_int32 (uri, "connectTimeoutMS", 3000);
+   server_count = test_framework_server_count ();
+   callbacks = heartbeat_callbacks ();
 
    if (pooled) {
       pool = mongoc_client_pool_new (uri);
+      mongoc_client_pool_set_apm_callbacks (pool, callbacks, &checks);
       test_framework_set_pool_ssl_opts (pool);
       client = mongoc_client_pool_pop (pool);
 
+      /* wait for all nodes to be scanned. */
+      WAIT_UNTIL (checks_cmp (&checks, "n_succeeded", '=', server_count));
+
       /* background scanner complains about failed connection */
       capture_logs (true);
    } else {
@@ -395,6 +533,10 @@ _test_topology_invalidate_server (bool pooled)
       &client->cluster, fake_id, true, NULL, NULL, &error));
    bson_mutex_lock (&client->topology->mutex);
    sd = (mongoc_server_description_t *) mongoc_set_get (td->servers, fake_id);
+   /* A single threaded client, during reconnect, will scan ALL servers.
+    * When it receives a response from one of those nodes, showing that
+    * "fakeaddress" is not in the host list, it will remove the
+    * server description from the topology description. */
    if (!pooled && test_framework_is_replset ()) {
       BSON_ASSERT (!sd);
    } else {
@@ -419,6 +561,8 @@ _test_topology_invalidate_server (bool pooled)
    } else {
       mongoc_client_destroy (client);
    }
+   mongoc_apm_callbacks_destroy (callbacks);
+   checks_cleanup (&checks);
 }
 
 static void
@@ -438,21 +582,18 @@ test_invalid_cluster_node (void *ctx)
 {
    mongoc_client_pool_t *pool;
    mongoc_cluster_node_t *cluster_node;
-   mongoc_topology_scanner_node_t *scanner_node;
    bson_error_t error;
    mongoc_client_t *client;
    mongoc_cluster_t *cluster;
    mongoc_server_stream_t *server_stream;
-   int64_t scanner_node_ts;
    uint32_t id;
+   mongoc_server_description_t *sd;
 
    /* use client pool, this test is only valid when multi-threaded */
    pool = test_framework_client_pool_new ();
    client = mongoc_client_pool_pop (pool);
    cluster = &client->cluster;
 
-   _mongoc_usleep (100 * 1000);
-
    /* load stream into cluster */
    server_stream = mongoc_cluster_stream_for_reads (
       &client->cluster, NULL, NULL, NULL, &error);
@@ -465,24 +606,23 @@ test_invalid_cluster_node (void *ctx)
    BSON_ASSERT (cluster_node->stream);
 
    bson_mutex_lock (&client->topology->mutex);
-   scanner_node =
-      mongoc_topology_scanner_get_node (client->topology->scanner, id);
-   BSON_ASSERT (scanner_node);
-   ASSERT_CMPINT64 (cluster_node->timestamp, >, scanner_node->timestamp);
+   sd = mongoc_topology_description_server_by_id (
+      &client->topology->description, id, &error);
+   ASSERT_OR_PRINT (sd, error);
+   /* Both generations match, and are the first generation. */
+   ASSERT_CMPINT32 (cluster_node->generation, ==, 0);
+   ASSERT_CMPINT32 (sd->generation, ==, 0);
 
-   /* update the scanner node's timestamp */
-   _mongoc_usleep (1000 * 1000);
-   scanner_node_ts = scanner_node->timestamp = bson_get_monotonic_time ();
-   ASSERT_CMPINT64 (cluster_node->timestamp, <, scanner_node_ts);
-   _mongoc_usleep (1000 * 1000);
+   /* update the server's generation, simulating a connection pool clearing */
+   sd->generation++;
    bson_mutex_unlock (&client->topology->mutex);
 
-   /* cluster discards node and creates new one */
+   /* cluster discards node and creates new one with the current generation */
    server_stream = mongoc_cluster_stream_for_server (
       &client->cluster, id, true, NULL, NULL, &error);
    ASSERT_OR_PRINT (server_stream, error);
    cluster_node = (mongoc_cluster_node_t *) mongoc_set_get (cluster->nodes, id);
-   ASSERT_CMPINT64 (cluster_node->timestamp, >, scanner_node_ts);
+   ASSERT_CMPINT64 (cluster_node->generation, ==, 1);
 
    mongoc_server_stream_cleanup (server_stream);
    mongoc_client_pool_push (pool, client);
@@ -492,7 +632,6 @@ test_invalid_cluster_node (void *ctx)
 static void
 test_max_wire_version_race_condition (void *ctx)
 {
-   mongoc_topology_scanner_node_t *scanner_node;
    mongoc_server_description_t *sd;
    mongoc_database_t *database;
    mongoc_client_pool_t *pool;
@@ -529,14 +668,12 @@ test_max_wire_version_race_condition (void *ctx)
    id = server_stream->sd->id;
    mongoc_server_stream_cleanup (server_stream);
 
-   /* "disconnect": invalidate timestamp and reset server description */
-   scanner_node =
-      mongoc_topology_scanner_get_node (client->topology->scanner, id);
-   BSON_ASSERT (scanner_node);
-   scanner_node->timestamp = bson_get_monotonic_time ();
+   /* "disconnect": increment generation and reset server description */
+
    sd = (mongoc_server_description_t *) mongoc_set_get (
       client->topology->description.servers, id);
    BSON_ASSERT (sd);
+   sd->generation++;
    mongoc_server_description_reset (sd);
 
    /* new stream, ensure that we can still auth with cached wire version */
@@ -1013,8 +1150,7 @@ _test_server_removed_during_handshake (bool pooled)
                               mock_server_get_host_and_port (server));
 
    /* pretend to close a connection. does NOT affect server description yet */
-   mongoc_cluster_disconnect_node (
-      &client->cluster, 1, false /* invalidate */, NULL);
+   mongoc_cluster_disconnect_node (&client->cluster, 1);
    sd = mongoc_client_get_server_description (client, 1);
    /* still primary */
    ASSERT_CMPINT ((int) MONGOC_SERVER_RS_PRIMARY, ==, sd->type);
@@ -1191,57 +1327,6 @@ test_add_and_scan_failure (void)
 }
 
 
-typedef struct {
-   int n_started;
-   int n_succeeded;
-   int n_failed;
-} checks_t;
-
-
-static void
-check_started (const mongoc_apm_server_heartbeat_started_t *event)
-{
-   checks_t *c;
-
-   c = (checks_t *) mongoc_apm_server_heartbeat_started_get_context (event);
-   c->n_started++;
-}
-
-
-static void
-check_succeeded (const mongoc_apm_server_heartbeat_succeeded_t *event)
-{
-   checks_t *c;
-
-   c = (checks_t *) mongoc_apm_server_heartbeat_succeeded_get_context (event);
-   c->n_succeeded++;
-}
-
-
-static void
-check_failed (const mongoc_apm_server_heartbeat_failed_t *event)
-{
-   checks_t *c;
-
-   c = (checks_t *) mongoc_apm_server_heartbeat_failed_get_context (event);
-   c->n_failed++;
-}
-
-
-static mongoc_apm_callbacks_t *
-heartbeat_callbacks (void)
-{
-   mongoc_apm_callbacks_t *callbacks;
-
-   callbacks = mongoc_apm_callbacks_new ();
-   mongoc_apm_set_server_heartbeat_started_cb (callbacks, check_started);
-   mongoc_apm_set_server_heartbeat_succeeded_cb (callbacks, check_succeeded);
-   mongoc_apm_set_server_heartbeat_failed_cb (callbacks, check_failed);
-
-   return callbacks;
-}
-
-
 static future_t *
 future_command (mongoc_client_t *client, bson_error_t *error)
 {
@@ -1281,7 +1366,7 @@ has_known_server (mongoc_client_t *client)
 static void
 _test_ismaster_retry_single (bool hangup, int n_failures)
 {
-   checks_t checks = {0};
+   checks_t checks;
    mongoc_apm_callbacks_t *callbacks;
    mock_server_t *server;
    mongoc_uri_t *uri;
@@ -1292,6 +1377,7 @@ _test_ismaster_retry_single (bool hangup, int n_failures)
    bson_error_t error;
    int64_t t;
 
+   checks_init (&checks);
    server = mock_server_new ();
    mock_server_run (server);
    uri = mongoc_uri_copy (mock_server_get_uri (server));
@@ -1367,13 +1453,14 @@ _test_ismaster_retry_single (bool hangup, int n_failures)
    mock_server_destroy (server);
    bson_free (ismaster);
    mongoc_apm_callbacks_destroy (callbacks);
+   checks_cleanup (&checks);
 }
 
 
 static void
 _test_ismaster_retry_pooled (bool hangup, int n_failures)
 {
-   checks_t checks = {0};
+   checks_t checks;
    mongoc_apm_callbacks_t *callbacks;
    mock_server_t *server;
    mongoc_uri_t *uri;
@@ -1383,9 +1470,9 @@ _test_ismaster_retry_pooled (bool hangup, int n_failures)
    future_t *future;
    request_t *request;
    bson_error_t error;
-   int i;
    int64_t t;
 
+   checks_init (&checks);
    server = mock_server_new ();
    mock_server_run (server);
    uri = mongoc_uri_copy (mock_server_get_uri (server));
@@ -1408,15 +1495,18 @@ _test_ismaster_retry_pooled (bool hangup, int n_failures)
                                   " 'hosts': ['%s']}",
                                   mock_server_get_host_and_port (server));
 
+   /* As soon as the client is popped, background monitoring starts. */
+   request = mock_server_receives_ismaster (server);
+   mock_server_replies_simple (request, ismaster);
+   request_destroy (request);
+
    /* start a {foo: 1} command, handshake normally */
    future = future_command (client, &error);
 
-   /* one ismaster from the scanner, another to handshake the connection */
-   for (i = 0; i < 2; i++) {
-      request = mock_server_receives_ismaster (server);
-      mock_server_replies_simple (request, ismaster);
-      request_destroy (request);
-   }
+   /* Another ismaster to handshake the connection */
+   request = mock_server_receives_ismaster (server);
+   mock_server_replies_simple (request, ismaster);
+   request_destroy (request);
 
    /* the {foo: 1} command finishes */
    receives_command (server, future);
@@ -1433,11 +1523,18 @@ _test_ismaster_retry_pooled (bool hangup, int n_failures)
    /* retry immediately (for testing, "immediately" means less than 250ms */
    request = mock_server_receives_ismaster (server);
    ASSERT_CMPINT64 (bson_get_monotonic_time () - t, <, (int64_t) 250 * 1000);
+   /* The server is marked as Unknown, but immediately rescanned. This behavior
+    * comes from the server monitoring spec:
+    * "To handle the case that the server is truly down, the monitor makes the
+    * server unselectable by marking it Unknown. To handle the case of a
+    * transient network glitch or restart, the monitor immediately runs the next
+    * check without waiting".
+    */
+   BSON_ASSERT (!has_known_server (client));
    if (n_failures == 2) {
       if (hangup) {
          mock_server_hangs_up (request);
       }
-      BSON_ASSERT (!has_known_server (client));
    } else {
       mock_server_replies_simple (request, ismaster);
       WAIT_UNTIL (has_known_server (client));
@@ -1445,9 +1542,9 @@ _test_ismaster_retry_pooled (bool hangup, int n_failures)
 
    request_destroy (request);
 
-   WAIT_UNTIL (checks.n_succeeded == 3 - n_failures);
-   WAIT_UNTIL (checks.n_failed == n_failures);
-   ASSERT_CMPINT (checks.n_started, ==, 3);
+   WAIT_UNTIL (checks_cmp (&checks, "n_succeeded", '=', 3 - n_failures));
+   WAIT_UNTIL (checks_cmp (&checks, "n_failed", '=', n_failures));
+   BSON_ASSERT (checks_cmp (&checks, "n_started", '=', 3));
 
    mongoc_client_pool_push (pool, client);
    mongoc_client_pool_destroy (pool);
@@ -1455,6 +1552,7 @@ _test_ismaster_retry_pooled (bool hangup, int n_failures)
    mock_server_destroy (server);
    bson_free (ismaster);
    mongoc_apm_callbacks_destroy (callbacks);
+   checks_cleanup (&checks);
 }
 
 
@@ -1666,8 +1764,7 @@ test_cluster_time_updated_during_handshake ()
                               cluster_time);
 
    /* remove the node from the cluster to trigger an ismaster handshake. */
-   mongoc_cluster_disconnect_node (
-      &client->cluster, 1, false /* invalidate */, NULL);
+   mongoc_cluster_disconnect_node (&client->cluster, 1);
 
    /* opens new stream and does an ismaster handshake (in pooled mode only). */
    r = mongoc_client_command_simple (
@@ -1684,38 +1781,6 @@ test_cluster_time_updated_during_handshake ()
    mongoc_uri_destroy (uri);
 }
 
-/* returns the last time the topology completed a full scan. */
-static int64_t
-_get_last_scan (mongoc_client_t *client)
-{
-   int64_t last_scan;
-   mongoc_topology_t *topology = client->topology;
-   bson_mutex_lock (&topology->mutex);
-   last_scan = topology->last_scan;
-   bson_mutex_unlock (&topology->mutex);
-   return last_scan;
-}
-
-typedef struct {
-   int64_t when_transitioned_to_unknown;
-   int64_t server_id;
-} request_scan_error_ctx_t;
-
-static void
-_server_changed (const mongoc_apm_server_changed_t *event)
-{
-   const mongoc_server_description_t *sd;
-   request_scan_error_ctx_t *ctx;
-
-   ctx = (request_scan_error_ctx_t *) mongoc_apm_server_changed_get_context (
-      event);
-   sd = mongoc_apm_server_changed_get_new_description (event);
-   if (sd->type == MONGOC_SERVER_UNKNOWN) {
-      ctx->when_transitioned_to_unknown = bson_get_monotonic_time ();
-      ctx->server_id = sd->id;
-   }
-}
-
 /* test that when a command receives a "not master" or "node is recovering"
  * error that the client takes the appropriate action:
  * - a pooled client should mark the server as unknown and request a full scan
@@ -1739,17 +1804,28 @@ _test_request_scan_on_error (bool pooled,
    bson_error_t error = {0};
    future_t *future = NULL;
    request_t *request;
-   const int64_t minHBMS = 50;
-   int64_t last_scan = 0;
-   mongoc_read_prefs_t *read_prefs;
+   const int64_t minHBMS = 10;
+   int64_t ping_started_usec = 0;
    mongoc_apm_callbacks_t *callbacks;
-   request_scan_error_ctx_t ctx = {0};
+   checks_t checks;
+   mongoc_server_description_t *sd;
+   uint32_t primary_id;
+   mongoc_read_prefs_t *read_prefs;
+
+   MONGOC_DEBUG ("pooled? %d", (int) pooled);
+   MONGOC_DEBUG ("err_response %s", err_response);
+   MONGOC_DEBUG ("should_scan %d, should_mark_unknown: %d",
+                 (int) should_scan,
+                 (int) should_mark_unknown);
+   MONGOC_DEBUG ("server_error %s", server_err);
+
+   checks_init (&checks);
+   read_prefs = mongoc_read_prefs_new (MONGOC_READ_PRIMARY_PREFERRED);
 
    primary = mock_server_new ();
    secondary = mock_server_new ();
    mock_server_run (primary);
    mock_server_run (secondary);
-   read_prefs = mongoc_read_prefs_new (MONGOC_READ_PRIMARY_PREFERRED);
 
    RS_RESPONSE_TO_ISMASTER (primary, 6, true, false, primary, secondary);
    RS_RESPONSE_TO_ISMASTER (secondary, 6, false, false, primary, secondary);
@@ -1768,31 +1844,34 @@ _test_request_scan_on_error (bool pooled,
       topology = _mongoc_client_pool_get_topology (client_pool);
       /* set a small minHeartbeatFrequency, so scans don't block for 500ms. */
       topology->min_heartbeat_frequency_msec = minHBMS;
-      client = mongoc_client_pool_pop (client_pool);
-      /* upon popping a client, the background monitoring thread is started. */
-      /* wait for the initial server selection to finish. */
-      WAIT_UNTIL (_get_last_scan (client) > last_scan);
    } else {
-      mongoc_server_description_t *sd;
       client = mongoc_client_new_from_uri (uri);
       /* set a small minHeartbeatFrequency, so scans don't block for 500ms. */
       client->topology->min_heartbeat_frequency_msec = minHBMS;
-      sd = mongoc_client_select_server (client, false, NULL, &error);
-      ASSERT_OR_PRINT (sd, error);
-      mongoc_server_description_destroy (sd);
    }
-   mongoc_uri_destroy (uri);
-   /* now that the initial server selection is completed, record the time. */
-   last_scan = _get_last_scan (client);
-   /* listen for transition to UNKNOWN */
-   callbacks = mongoc_apm_callbacks_new ();
-   mongoc_apm_set_server_changed_cb (callbacks, _server_changed);
+
+   callbacks = heartbeat_callbacks ();
    if (pooled) {
-      mongoc_client_pool_set_apm_callbacks (client_pool, callbacks, &ctx);
+      mongoc_client_pool_set_apm_callbacks (client_pool, callbacks, &checks);
    } else {
-      mongoc_client_set_apm_callbacks (client, callbacks, &ctx);
+      mongoc_client_set_apm_callbacks (client, callbacks, &checks);
    }
    mongoc_apm_callbacks_destroy (callbacks);
+
+   if (pooled) {
+      client = mongoc_client_pool_pop (client_pool);
+      /* Scanning starts, wait for the initial scan. */
+      WAIT_UNTIL (checks_cmp (&checks, "n_succeeded", '=', 2));
+   }
+
+   sd = mongoc_client_select_server (client, true, NULL, &error);
+   ASSERT_OR_PRINT (sd, error);
+   primary_id = sd->id;
+   mongoc_server_description_destroy (sd);
+   BSON_ASSERT (checks_cmp (&checks, "n_succeeded", '=', 2));
+
+   mongoc_uri_destroy (uri);
+   ping_started_usec = bson_get_monotonic_time ();
    /* run a ping command on the primary. */
    future = future_client_command_simple (
       client, "db", tmp_bson ("{'ping': 1}"), read_prefs, &reply, &error);
@@ -1810,32 +1889,42 @@ _test_request_scan_on_error (bool pooled,
    future_destroy (future);
    bson_destroy (&reply);
 
+   sd = mongoc_client_get_server_description (client, primary_id);
    if (should_mark_unknown) {
-      mongoc_server_description_t *sd;
-      /* between sending the 'ping' command and returning, the server should
-       * have been marked as unknown. */
-      ASSERT_CMPINT64 (last_scan, <=, ctx.when_transitioned_to_unknown);
-      ASSERT_CMPINT64 (
-         ctx.when_transitioned_to_unknown, <=, bson_get_monotonic_time ());
-      sd = mongoc_client_get_server_description (client,
-                                                 (uint32_t) ctx.server_id);
-      /* check that the error on the server description matches the error
-       * message in the response. */
-      if (server_err) {
-         ASSERT_CMPSTR (server_err, sd->error.message);
+      BSON_ASSERT (checks_cmp (&checks, "n_unknowns", '=', 1));
+      /* background monitoring may have already overwritten the unknown server
+       * description if the scan was requested. */
+      if (pooled) {
+         if (sd->type == MONGOC_SERVER_UNKNOWN) {
+            if (server_err) {
+               ASSERT_CMPSTR (server_err, sd->error.message);
+            }
+         }
+      } else {
+         /* after the 'ping' command and returning, the server should
+            * have been marked as unknown. */
+         BSON_ASSERT (sd->type == MONGOC_SERVER_UNKNOWN);
+         ASSERT_CMPINT (sd->last_update_time_usec, >=, ping_started_usec);
+         ASSERT_CMPINT (
+            sd->last_update_time_usec, <=, bson_get_monotonic_time ());
+         /* check that the error on the server description matches the error
+         * message in the response. */
+         if (server_err) {
+            ASSERT_CMPSTR (server_err, sd->error.message);
+         }
       }
-      mongoc_server_description_destroy (sd);
    } else {
-      ASSERT_CMPINT64 (ctx.when_transitioned_to_unknown, ==, (int64_t) 0);
+      BSON_ASSERT (sd->type != MONGOC_SERVER_UNKNOWN);
    }
+   mongoc_server_description_destroy (sd);
 
    if (pooled) {
       if (should_scan) {
          /* a scan is requested immediately. wait for the scan to finish. */
-         WAIT_UNTIL (_get_last_scan (client) > last_scan);
+         WAIT_UNTIL (checks_cmp (&checks, "n_started", '=', 4));
       } else {
-         /* wait a short while to make sure no scan occurs. */
-         _mongoc_usleep (10 * 1000);
+         _mongoc_usleep (minHBMS * 2);
+         BSON_ASSERT (checks_cmp (&checks, "n_started", '=', 2));
       }
    } else {
       /* a single threaded client may mark the topology as stale. if a scan
@@ -1856,15 +1945,13 @@ _test_request_scan_on_error (bool pooled,
       BSON_ASSERT (future_get_bool (future));
       future_destroy (future);
       bson_destroy (&reply);
+      if (should_scan) {
+         BSON_ASSERT (checks_cmp (&checks, "n_started", '=', 4));
+      } else {
+         BSON_ASSERT (checks_cmp (&checks, "n_started", '=', 2));
+      }
    }
 
-   if (should_scan) {
-      ASSERT_CMPINT64 (last_scan, <, _get_last_scan (client));
-   } else {
-      ASSERT_CMPINT64 (last_scan, ==, _get_last_scan (client));
-   }
-
-   mongoc_read_prefs_destroy (read_prefs);
    if (pooled) {
       mongoc_client_pool_push (client_pool, client);
       mongoc_client_pool_destroy (client_pool);
@@ -1873,10 +1960,12 @@ _test_request_scan_on_error (bool pooled,
    }
    mock_server_destroy (primary);
    mock_server_destroy (secondary);
+   mongoc_read_prefs_destroy (read_prefs);
+   checks_cleanup (&checks);
 }
 
 static void
-test_last_server_removed_warning ()
+test_last_server_removed_warning (void)
 {
    mock_server_t *server;
    mongoc_client_t *client;
@@ -1938,10 +2027,22 @@ test_request_scan_on_error ()
                 false /* should_scan */,
                 true /* should_mark_unknown */,
                 "node is recovering");
+   /* Test that "not master or secondary" is considered a "node is recovering"
+    * error, not a "not master" error. */
+   TEST_SINGLE ("{'ok': 0, 'errmsg': 'not master or secondary'}",
+                false /* should_scan */,
+                true /* should_mark_unknown */,
+                "not master or secondary");
    TEST_POOLED ("{'ok': 0, 'errmsg': 'node is recovering'}",
                 true /* should_scan */,
                 true /* should_mark_unknown */,
                 "node is recovering");
+   /* Test that "not master or secondary" is considered a "node is recovering"
+    * error, not a "not master" error. */
+   TEST_POOLED ("{'ok': 0, 'errmsg': 'not master or secondary'}",
+                true /* should_scan */,
+                true /* should_mark_unknown */,
+                "not master or secondary");
    TEST_BOTH ("{'ok': 0, 'errmsg': 'random error'}",
               false /* should_scan */,
               false /* should_mark_unknown */,
@@ -1967,12 +2068,18 @@ test_request_scan_on_error ()
                 true /* should_scan */,
                 true /* should_mark_unknown */,
                 NULL /* server_err */);
-   /* with a "not master" error code but a "node is recovery" message, the error
-    * code takes precedence */
-   TEST_BOTH ("{'ok': 0, 'code': 10107, 'errmsg': 'node is recovering'}",
-              true /* should_scan */,
-              true /* should_mark_unknown */,
-              "node is recovering");
+   /* with a "not master" error code but a "node is recovery" message, it is
+    * considered a "node is recovering" error */
+   TEST_SINGLE ("{'ok': 0, 'code': 10107, 'errmsg': 'node is recovering'}",
+                false /* should_scan */,
+                true /* should_mark_unknown */,
+                "node is recovering");
+   /* with a "not master" error code but a "node is recovery" message, it is
+    * considered a "node is recovering" error */
+   TEST_POOLED ("{'ok': 0, 'code': 10107, 'errmsg': 'node is recovering'}",
+                true /* should_scan */,
+                true /* should_mark_unknown */,
+                "node is recovering");
    /* write concern errors are also checked. */
    TEST_BOTH ("{'ok': 1, 'writeConcernError': { 'errmsg': 'not master' }}",
               true, /* should_scan */
@@ -1988,7 +2095,113 @@ test_request_scan_on_error ()
 #undef TEST_SINGLE
 }
 
+/* Test that the issue described in CDRIVER-3625 is fixed.
+ * A slow-to-respond server should not block the scan of other servers
+ * in background monitoring.
+ */
+static void
+test_slow_server_pooled (void)
+{
+   mock_server_t *primary;
+   mock_server_t *secondary;
+   char *ismaster_common;
+   char *ismaster_primary;
+   char *ismaster_secondary;
+   mongoc_read_prefs_t *prefs_secondary;
+   mongoc_client_pool_t *pool;
+   mongoc_client_t *client;
+   mongoc_uri_t *uri;
+   mongoc_apm_callbacks_t *callbacks;
+   request_t *request;
+   checks_t checks;
+   bool ret;
+   bson_error_t error;
 
+   checks_init (&checks);
+   primary = mock_server_new ();
+   secondary = mock_server_new ();
+
+   mock_server_run (primary);
+   mock_server_run (secondary);
+
+   mock_server_autoresponds (primary, auto_ping, NULL, NULL);
+   mock_server_autoresponds (secondary, auto_ping, NULL, NULL);
+
+   ismaster_common = bson_strdup_printf (
+      "{'ok': 1, 'setName': 'rs', 'hosts': ['%s', '%s'], 'maxWireVersion': %d",
+      mock_server_get_host_and_port (primary),
+      mock_server_get_host_and_port (secondary),
+      WIRE_VERSION_MAX);
+   ismaster_primary = bson_strdup_printf (
+      "%s, 'ismaster': true, 'secondary': false }", ismaster_common);
+   ismaster_secondary = bson_strdup_printf (
+      "%s, 'ismaster': false, 'secondary': true }", ismaster_common);
+
+   /* Primary responds immediately, but secondary does not. */
+   mock_server_auto_ismaster (primary, ismaster_primary);
+
+   uri = mongoc_uri_copy (mock_server_get_uri (primary));
+   /* Do not connect as topology type Single, so the client pool discovers the
+    * secondary. */
+   mongoc_uri_set_option_as_bool (uri, MONGOC_URI_DIRECTCONNECTION, false);
+   mongoc_uri_set_option_as_int32 (
+      uri, MONGOC_URI_SERVERSELECTIONTIMEOUTMS, 500);
+
+   pool = mongoc_client_pool_new (uri);
+   callbacks = heartbeat_callbacks ();
+   mongoc_client_pool_set_apm_callbacks (pool, callbacks, &checks);
+
+   /* Set a shorter heartbeat frequencies for faster responses. */
+   _mongoc_client_pool_get_topology (pool)->description.heartbeat_msec = 10;
+   _mongoc_client_pool_get_topology (pool)->min_heartbeat_frequency_msec = 10;
+
+   client = mongoc_client_pool_pop (pool);
+   /* As soon as a client is popped, background scanning starts.
+    * Wait for two scans of the primary. */
+   WAIT_UNTIL (checks_cmp (&checks, "n_started", '>', 1));
+
+   request = mock_server_receives_ismaster (secondary);
+
+   /* A command to the primary succeeds. */
+   ret = mongoc_client_command_simple (
+      client, "admin", tmp_bson ("{'ping': 1}"), NULL, NULL, &error);
+   ASSERT_OR_PRINT (ret, error);
+
+   /* A command to the secondary fails. */
+   prefs_secondary = mongoc_read_prefs_new (MONGOC_READ_SECONDARY);
+   ret = mongoc_client_command_simple (
+      client, "admin", tmp_bson ("{'ping': 1}"), prefs_secondary, NULL, &error);
+   ASSERT_ERROR_CONTAINS (error,
+                          MONGOC_ERROR_SERVER_SELECTION,
+                          MONGOC_ERROR_SERVER_SELECTION_FAILURE,
+                          "expired");
+   BSON_ASSERT (!ret);
+
+   /* Set up an auto responder so future ismasters on the secondary do not
+    * block until connectTimeoutMS. Otherwise, the shutdown sequence will be
+    * blocked for connectTimeoutMS. */
+   mock_server_auto_ismaster (secondary, ismaster_secondary);
+   /* Respond to the first ismaster. */
+   mock_server_replies_simple (request, ismaster_secondary);
+   request_destroy (request);
+
+   /* Now a command to the secondary succeeds. */
+   ret = mongoc_client_command_simple (
+      client, "admin", tmp_bson ("{'ping': 1}"), prefs_secondary, NULL, &error);
+   ASSERT_OR_PRINT (ret, error);
+
+   mongoc_read_prefs_destroy (prefs_secondary);
+   mongoc_client_pool_push (pool, client);
+   mongoc_apm_callbacks_destroy (callbacks);
+   mongoc_client_pool_destroy (pool);
+   mongoc_uri_destroy (uri);
+   bson_free (ismaster_secondary);
+   bson_free (ismaster_primary);
+   bson_free (ismaster_common);
+   mock_server_destroy (secondary);
+   mock_server_destroy (primary);
+   checks_cleanup (&checks);
+}
 void
 test_topology_install (TestSuite *suite)
 {
@@ -2067,10 +2280,16 @@ test_topology_install (TestSuite *suite)
                       NULL,
                       NULL,
                       test_framework_skip_if_offline);
-   TestSuite_AddMockServerTest (
-      suite, "/Topology/connect_timeout/succeed", test_select_after_timeout);
-   TestSuite_AddMockServerTest (
-      suite, "/Topology/try_once/succeed", test_select_after_try_once);
+   _TestSuite_AddMockServerTest (suite,
+                                 "/Topology/connect_timeout/succeed",
+                                 test_select_after_timeout,
+                                 test_framework_skip_if_time_sensitive,
+                                 NULL);
+   _TestSuite_AddMockServerTest (suite,
+                                 "/Topology/try_once/succeed",
+                                 test_select_after_try_once,
+                                 test_framework_skip_if_time_sensitive,
+                                 NULL);
    TestSuite_AddLive (
       suite, "/Topology/invalid_server_id", test_invalid_server_id);
    TestSuite_AddMockServerTest (suite,
@@ -2135,4 +2354,6 @@ test_topology_install (TestSuite *suite)
    TestSuite_AddMockServerTest (suite,
                                 "/Topology/last_server_removed_warning",
                                 test_last_server_removed_warning);
+   TestSuite_AddMockServerTest (
+      suite, "/Topology/slow_server/pooled", test_slow_server_pooled);
 }
